var __extends = this && this.__extends || function () {
  var _extendStatics = function extendStatics(d, b) {
    _extendStatics = Object.setPrototypeOf || {
      __proto__: []
    } instanceof Array && function (d, b) {
      d.__proto__ = b;
    } || function (d, b) {
      for (var p in b) {
        if (b.hasOwnProperty(p)) d[p] = b[p];
      }
    };

    return _extendStatics(d, b);
  };

  return function (d, b) {
    _extendStatics(d, b);

    function __() {
      this.constructor = d;
    }

    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
  };
}();

var __assign = this && this.__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) {
        if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
      }
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

import React from 'react';
import classNames from 'classnames';
import throttleWithRAF from '../utils/throttleWithRAF';
import addEventListener from '../utils/addEventListener';
import { getOffset, getScrollValue } from '../utils/get';
import PortalAffix from './PortalAffix';

var Affix =
/** @class */
function (_super) {
  __extends(Affix, _super);

  function Affix(props) {
    var _this = _super.call(this, props) || this;

    _this.prefixCls = 're-affix';
    _this.wrapRef = React.createRef();
    _this.customRef = React.createRef();

    _this.onHandlerScroll = function () {
      var _a = _this.props,
          offsetTop = _a.offsetTop,
          onChange = _a.onChange,
          onScroll = _a.onScroll,
          renderAffix = _a.renderAffix,
          allWidth = _a.allWidth,
          fadeEffect = _a.fadeEffect,
          getFixedStatus = _a.getFixedStatus;
      var element = _this.wrapRef.current;

      if (!element) {
        return;
      }

      var scrollTop = getScrollValue().top;
      var parentElement = element.parentNode; // 元素信息

      var positionInfo = _this.isFixed && !renderAffix ? getOffset(parentElement) : getOffset(element);
      var currentFixed = getFixedStatus({
        positionInfo: positionInfo,
        scrollTop: scrollTop,
        offsetTop: offsetTop
      });

      if (_this.isFixed && !currentFixed) {
        _this.isFixed = false;

        if (renderAffix || fadeEffect) {
          _this.setState({
            visible: false
          });
        } else {
          element.style.position = '';
          element.style.top = '';
          element.style.left = '';

          if (allWidth) {
            element.style.width = '';
          }

          parentElement.style.width = '';
          parentElement.style.height = '';
        }

        onChange(false);
      } else if (!_this.isFixed && currentFixed) {
        _this.isFixed = true;
        var documentWidth = document.body.offsetWidth;

        if (renderAffix || fadeEffect) {
          _this.setState({
            visible: true,
            left: allWidth ? '0px' : positionInfo.left + "px",
            top: offsetTop + "px",
            width: allWidth ? documentWidth + "px" : ''
          });
        } else {
          element.style.position = 'fixed';
          element.style.top = offsetTop + "px";
          element.style.left = allWidth ? '0px' : positionInfo.left + "px";

          if (allWidth) {
            element.style.width = documentWidth + "px";
          }

          parentElement.style.width = positionInfo.width + "px";
          parentElement.style.height = positionInfo.height + "px";
        }

        onChange(true);
      }

      if (onScroll) {
        onScroll(__assign(__assign({}, positionInfo), {
          scrollTop: scrollTop,
          isFixed: _this.isFixed
        }));
      }
    };

    _this.state = {
      visible: false,
      left: '',
      top: '',
      width: ''
    };
    return _this;
  }

  Affix.prototype.componentDidMount = function () {
    this.onHandlerScroll();
    this.scrollHandler = addEventListener(window, 'scroll', throttleWithRAF(this.onHandlerScroll));
  };

  Affix.prototype.componentWillUnmount = function () {
    if (this.scrollHandler) {
      this.scrollHandler.remove();
      this.scrollHandler = null;
    }
  };

  Affix.prototype.renderContent = function () {
    var _a = this.props,
        fadeEffect = _a.fadeEffect,
        renderAffix = _a.renderAffix,
        className = _a.className,
        children = _a.children;
    var _b = this.state,
        visible = _b.visible,
        left = _b.left,
        top = _b.top,
        width = _b.width;

    if ((fadeEffect || renderAffix) && (visible || this.customRef.current)) {
      return React.createElement(PortalAffix, {
        top: top,
        width: width,
        left: left,
        fadeEffect: fadeEffect,
        prefixCls: this.prefixCls,
        className: className,
        visible: visible,
        ref: this.customRef
      }, renderAffix ? renderAffix() : children);
    }

    return null;
  };

  Affix.prototype.render = function () {
    var _a = this.props,
        className = _a.className,
        children = _a.children; // 这里需要两层div 包裹，最外层的是在固定时，防止页面抖动

    return React.createElement("div", null, React.createElement("div", {
      ref: this.wrapRef,
      className: classNames(this.prefixCls, className)
    }, children), this.renderContent());
  };

  Affix.defaultProps = {
    offsetTop: 0,
    allWidth: false,
    onChange: function onChange() {},
    getFixedStatus: function getFixedStatus(_a) {
      var positionInfo = _a.positionInfo,
          scrollTop = _a.scrollTop,
          offsetTop = _a.offsetTop;
      return scrollTop > positionInfo.top - offsetTop;
    }
  };
  return Affix;
}(React.Component);

export default Affix;