var __extends = this && this.__extends || function () {
  var _extendStatics = function extendStatics(d, b) {
    _extendStatics = Object.setPrototypeOf || {
      __proto__: []
    } instanceof Array && function (d, b) {
      d.__proto__ = b;
    } || function (d, b) {
      for (var p in b) {
        if (b.hasOwnProperty(p)) d[p] = b[p];
      }
    };

    return _extendStatics(d, b);
  };

  return function (d, b) {
    _extendStatics(d, b);

    function __() {
      this.constructor = d;
    }

    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
  };
}();

import React from 'react';
import classNames from 'classnames';
import Transition from './transition';
import { getElementSize } from '../utils/get';
import { isTouchScreen } from '../utils/string';
import addEventListener from '../utils/addEventListener';
import throttleWithRAF from '../utils/throttleWithRAF';

var Carousel =
/** @class */
function (_super) {
  __extends(Carousel, _super);

  function Carousel(props) {
    var _this = _super.call(this, props) || this;

    _this.prefixCls = 're-carousel';
    _this.sliderWidth = 0;
    _this.startX = 0;
    _this.prevX = 0;
    _this.moveX = 0;
    _this.currentX = 0;
    _this.isDirectionLeft = true;
    _this.sliderRef = React.createRef();
    _this.autoplayTimer = null;
    _this.fadeEffectTimer = null;

    _this.getChildCount = function () {
      return React.Children.count(_this.props.children);
    };

    _this.getTrackWidth = function () {
      return _this.sliderWidth * _this.getChildCount();
    };

    _this.init = function (sliderEl) {
      var element = sliderEl || _this.getSliderDomNode();

      if (!element) {
        return;
      }

      var currentPage = _this.state.currentPage;
      var sliderWidth = _this.props.sliderWidth || getElementSize(element.parentNode).width;

      var length = _this.getChildCount();

      _this.sliderWidth = sliderWidth;
      var childList = Array.from(element.children);
      childList.forEach(function (child) {
        child.style.width = sliderWidth + "px";
      });
      element.style.width = sliderWidth * length + "px";
      _this.prevX = -(currentPage - 1) * sliderWidth;

      _this.onBeforeChange(currentPage);

      _this.setTrackStyles(element, _this.prevX);

      _this.onAutoplay();
    };

    _this.getSliderDomNode = function () {
      if (_this.sliderRef && _this.sliderRef.current) {
        return _this.sliderRef.current;
      }

      return null;
    };

    _this.setTrackStyles = function (element, translateX, transition) {
      element.style.transition = '';

      if (_this.props.fadeEffect) {
        clearTimeout(_this.fadeEffectTimer);
        element.style.visibility = 'none';
        element.style.opacity = '0';
        _this.fadeEffectTimer = setTimeout(function () {
          element.style.transition = Transition.Opacity;
          element.style.opacity = '1';
          element.style.visibility = '';
        });
      } else {
        element.style.transition = transition || Transition.TransformEaseInOut;
      }

      element.style.transform = "translateX(" + translateX + "px)";
    };

    _this.changeSlide = function (nextPage, transition, isTouchMove) {
      clearTimeout(_this.autoplayTimer);

      var sliderEl = _this.getSliderDomNode();

      if (sliderEl && _this.sliderWidth) {
        var trackWidth = _this.getTrackWidth();

        var previewWidth = getElementSize(sliderEl.parentNode).width; // 如果父级宽度大于整个列表宽度，则移动时，恢复原样

        if (trackWidth <= previewWidth) {
          _this.setTrackStyles(sliderEl, 0, transition);

          return;
        }

        var childCount = _this.getChildCount();

        if ((nextPage - 1) * _this.sliderWidth + previewWidth > trackWidth || nextPage > childCount) {
          // 由于不是无穷尽轮播图，在手势切换下，不需要切换到最初位置
          nextPage = isTouchMove ? childCount - Math.ceil(previewWidth / _this.sliderWidth) + 1 : 1;
        } else if (nextPage < 1) {
          nextPage = isTouchMove ? 1 : childCount - Math.ceil(previewWidth / _this.sliderWidth) + 1;
        }

        _this.prevX = -(nextPage - 1) * _this.sliderWidth;

        _this.onBeforeChange(nextPage);

        _this.setTrackStyles(sliderEl, _this.prevX, transition);
      }

      _this.setState({
        currentPage: nextPage
      }, function () {
        _this.onAutoplay();
      });
    };

    _this.next = function (transition) {
      _this.changeSlide(_this.state.currentPage + 1, transition);
    };

    _this.prev = function (transition) {
      _this.changeSlide(_this.state.currentPage - 1, transition);
    };

    _this.goTo = function (nextIndex, transition, isTouchMove) {
      _this.changeSlide(nextIndex, transition, isTouchMove);
    };

    _this.onBeforeChange = function (currentPage) {
      if (typeof _this.props.onBeforeChange === 'function') {
        _this.props.onBeforeChange(currentPage);
      }
    };

    _this.onAfterChange = function () {
      if (typeof _this.props.onAfterChange === 'function') {
        _this.props.onAfterChange(_this.state.currentPage);
      }
    };

    _this.onClick = function () {
      if (isTouchScreen()) {
        return;
      }

      if (_this.props.onClick) {
        _this.props.onClick(_this.state.currentPage);
      }
    };

    _this.onSwipeStart = function (event) {
      if (!_this.props.controllable || _this.props.fadeEffect) {
        return;
      }

      event.preventDefault();
      clearTimeout(_this.autoplayTimer);
      _this.currentX = 0;
      _this.startX = event.touches ? event.touches[0].pageX : event.clientX;

      var sliderEl = _this.getSliderDomNode();

      if (!sliderEl) {
        return;
      }

      sliderEl.style.transition = 'none';
    };

    _this.onSwipeMove = function (event) {
      if (!_this.props.controllable || _this.props.fadeEffect) {
        return;
      }

      event.preventDefault();
      _this.currentX = event.touches ? event.touches[0].pageX : event.clientX;
      _this.moveX = _this.prevX + _this.currentX - _this.startX;
      _this.isDirectionLeft = _this.prevX - _this.startX <= _this.prevX - _this.currentX;

      var sliderEl = _this.getSliderDomNode();

      if (!sliderEl) {
        return;
      }

      sliderEl.style.transform = "translateX(" + _this.moveX + "px)";
    };

    _this.onSwipeEnd = function (event) {
      var _a = _this.props,
          onClick = _a.onClick,
          controllable = _a.controllable,
          fadeEffect = _a.fadeEffect,
          sensitivity = _a.sensitivity;

      if (!controllable || fadeEffect) {
        if (onClick) {
          onClick(_this.state.currentPage);
        }

        return;
      }

      if (_this.currentX === 0 || Math.abs(_this.startX - _this.currentX) < sensitivity) {
        if (onClick) {
          onClick(_this.state.currentPage);
        }
      }

      var transition = Transition.TransformEaseOut;
      event.preventDefault();

      if (_this.moveX >= 0) {
        _this.goTo(1, transition, true);

        return;
      }

      var trackWidth = _this.getTrackWidth();

      var moveX = Math.abs(_this.moveX);

      if (moveX >= trackWidth - _this.sliderWidth) {
        _this.goTo(_this.getChildCount(), transition, true);

        return;
      }

      var currentPage = _this.state.currentPage;
      var skipOne = moveX % _this.sliderWidth >= _this.props.sensitivity ? 1 : 0;
      var skipCount = Math.floor(Math.abs(moveX - Math.abs(_this.prevX)) / _this.sliderWidth);

      if (_this.isDirectionLeft) {
        currentPage = currentPage + skipCount + skipOne;
      } else {
        currentPage -= skipCount + skipOne;
      }

      _this.goTo(currentPage, transition, true);
    };

    _this.state = {
      currentPage: props.defaultValue,
      initialized: false
    };
    return _this;
  }

  Carousel.prototype.componentDidMount = function () {
    this.setState({
      initialized: true
    });
    this.init();
  };

  Carousel.prototype.componentDidUpdate = function (prevProps) {
    var _this = this;

    var children = this.props.children;

    if (React.Children.count(children) !== React.Children.count(prevProps.children)) {
      this.init();
    } // 加入 resize 监听


    if (this.props.monitorWindowResize && !this.resizeHandler) {
      this.resizeHandler = addEventListener(window, 'resize', throttleWithRAF(function () {
        return _this.init();
      }));
    }
  };

  Carousel.prototype.componentWillUnmount = function () {
    clearTimeout(this.autoplayTimer);
    clearTimeout(this.fadeEffectTimer);

    if (this.resizeHandler) {
      this.resizeHandler.remove();
      this.resizeHandler = null;
    }
  };

  Carousel.prototype.onAutoplay = function () {
    var _this = this;

    if (!this.props.autoplay || this.getChildCount() <= 1) {
      return;
    }

    clearTimeout(this.autoplayTimer);
    this.autoplayTimer = setTimeout(function () {
      _this.changeSlide(_this.state.currentPage + 1);
    }, this.props.delay);
  };

  Carousel.prototype.getDefaultStyles = function () {
    var result = {
      trackStyles: {},
      sliderStyles: null
    };
    var _a = this.props,
        height = _a.height,
        sliderWidth = _a.sliderWidth;

    if (height) {
      result.trackStyles.height = height;
    }

    if (sliderWidth) {
      var childCount = this.getChildCount();
      result.trackStyles.width = childCount * sliderWidth + "px";
      result.sliderStyles = {
        width: sliderWidth + "px"
      };
    }

    return result;
  };

  Carousel.prototype.render = function () {
    var _a;

    var cls = this.prefixCls;
    var _b = this.props,
        className = _b.className,
        children = _b.children,
        pagination = _b.pagination;

    var _c = this.getDefaultStyles(),
        trackStyles = _c.trackStyles,
        sliderStyles = _c.sliderStyles;

    return React.createElement("div", {
      className: classNames(cls, className, (_a = {}, _a[cls + "-init"] = this.state.initialized, _a))
    }, React.createElement("div", {
      className: cls + "-list"
    }, React.createElement("div", {
      style: trackStyles,
      className: cls + "-track",
      ref: this.sliderRef,
      onTouchStart: this.onSwipeStart,
      onTouchMove: this.onSwipeMove,
      onTouchEnd: this.onSwipeEnd,
      onClick: this.onClick,
      onTransitionEnd: this.onAfterChange
    }, React.Children.map(children, function (child, index) {
      return React.createElement("div", {
        key: index,
        className: cls + "-slider",
        style: sliderStyles
      }, child);
    }))), typeof pagination === 'function' ? pagination(this) : pagination);
  }; // 默认值


  Carousel.defaultProps = {
    defaultValue: 1,
    className: '',
    sensitivity: 30,
    autoplay: false,
    delay: 3000,
    height: '',
    controllable: true,
    fadeEffect: false,
    monitorWindowResize: true
  };
  return Carousel;
}(React.Component);

export default Carousel;