var __assign = this && this.__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) {
        if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
      }
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

import React from 'react';
import ReactDOM from 'react-dom';
import classNames from 'classnames';
import { canUseDOM } from '../utils/string';
import Modal from '../modal';
var prefixCls = 're-confirm';
var defaultConfig = {
  hiddenCancel: false,
  hiddenOk: false,
  cancelText: 'Cancel',
  cancelClassName: prefixCls + "-cancel",
  okText: 'Confirm',
  okClassName: prefixCls + "-ok",
  content: 'Are you sure?',
  width: '3.6rem',
  placement: 'center',
  visible: true,
  onOk: function onOk() {},
  onCancel: function onCancel() {}
};
export default function Confirm(config) {
  if (!canUseDOM) {
    throw new Error('Confirm use client!');
  }

  var div = document.createElement('div');
  document.body.appendChild(div);
  var currentConfig = defaultConfig;

  function updateCurrentConfig(config) {
    var cancelClassName = classNames(currentConfig.cancelClassName, config.cancelClassName);
    var okClassName = classNames(currentConfig.okClassName, config.okClassName);
    currentConfig = __assign(__assign(__assign({}, currentConfig), config), {
      cancelClassName: cancelClassName,
      okClassName: okClassName,
      onDestroy: onDestroy
    });
  }

  updateCurrentConfig(config);

  function close(type) {
    if (type === void 0) {
      type = 'cancel';
    }

    currentConfig = __assign(__assign({}, currentConfig), {
      visible: false
    });
    render(currentConfig);

    if (type === 'ok') {
      currentConfig.onOk();
    } else {
      currentConfig.onCancel();
    }
  }

  function onDestroy() {
    var unmountResult = ReactDOM.unmountComponentAtNode(div);

    if (unmountResult && div.parentNode) {
      try {
        div.parentNode.removeChild(div);
      } catch (e) {}
    }
  }

  function onVisible() {
    close('cancel');
  }

  function onConfirm() {
    close('ok');
  }

  function render(props) {
    ReactDOM.render(React.createElement(Modal, __assign({}, props, {
      className: classNames(prefixCls + "-wrap", props.className),
      visible: props.visible,
      onClose: onVisible
    }), props.content, props.hiddenCancel && props.hiddenOk ? null : React.createElement("div", {
      className: prefixCls + "-btns"
    }, props.hiddenCancel ? null : React.createElement("button", {
      className: props.cancelClassName,
      onClick: onVisible
    }, props.cancelText), props.hiddenOk ? null : React.createElement("button", {
      className: props.okClassName,
      onClick: onConfirm
    }, props.okText))), div);
  } // 渲染组件


  render(currentConfig);
  return {
    destroy: close,
    forceUpdate: function forceUpdate(config) {
      updateCurrentConfig(config);
      render(currentConfig);
    }
  };
}