var __extends = this && this.__extends || function () {
  var _extendStatics = function extendStatics(d, b) {
    _extendStatics = Object.setPrototypeOf || {
      __proto__: []
    } instanceof Array && function (d, b) {
      d.__proto__ = b;
    } || function (d, b) {
      for (var p in b) {
        if (b.hasOwnProperty(p)) d[p] = b[p];
      }
    };

    return _extendStatics(d, b);
  };

  return function (d, b) {
    _extendStatics(d, b);

    function __() {
      this.constructor = d;
    }

    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
  };
}();

import React from 'react';
import classNames from 'classnames';
import { findDOMNode } from 'react-dom';
import Portal from '../portal';
import UseConfig from '../use-config';
import { getOffset } from '../utils/get';
import Mask from '../mask';

var Guide =
/** @class */
function (_super) {
  __extends(Guide, _super);

  function Guide(props) {
    var _this = _super.call(this, props) || this;

    _this.prefixCls = 're-guide';
    _this.timer = null;
    _this.imgRef = React.createRef(); // eslint-disable-next-line react/no-find-dom-node

    _this.getElement = function () {
      return findDOMNode(_this);
    };

    _this.changeVisible = function () {
      if (_this.timer) {
        clearTimeout(_this.timer);
      }

      var _a = _this.props,
          onClose = _a.onClose,
          expires = _a.expires;
      _this.timer = setTimeout(function () {
        onClose();
      }, expires);
    };

    _this.onClose = function () {
      if (_this.timer) {
        clearTimeout(_this.timer);
      }

      _this.props.onClose();
    };

    _this.state = {
      top: '0',
      left: '0'
    };
    return _this;
  }

  Guide.prototype.componentDidMount = function () {
    if (this.props.visible) {
      this.changeVisible();
      this.positionPointer();
    }
  };

  Guide.prototype.componentDidUpdate = function (prevProps) {
    if (prevProps.visible !== this.props.visible && this.props.visible) {
      this.changeVisible();
      this.positionPointer();
    }
  };

  Guide.prototype.componentWillUnmount = function () {
    if (this.timer) {
      clearTimeout(this.timer);
    }
  };

  Guide.prototype.positionPointer = function () {
    var _this = this;

    var _a = this.props,
        loadablePointer = _a.loadablePointer,
        img = _a.img,
        position = _a.position,
        visible = _a.visible,
        translateX = _a.translateX,
        translateY = _a.translateY;

    if (loadablePointer && visible) {
      var image_1 = new Image();
      image_1.src = img;

      image_1.onload = function () {
        var element = _this.getElement();

        if (element) {
          var _a = getOffset(element),
              left = _a.left,
              top_1 = _a.top,
              width = _a.width;

          var imgWidth = image_1.width;
          var imgHeight = image_1.height;
          var imgTop = top_1 - imgHeight + translateY + "px";

          if (position === 'left') {
            _this.setState({
              top: imgTop,
              left: left + translateX + "px"
            });

            return;
          }

          if (position === 'right') {
            _this.setState({
              top: imgTop,
              left: left + width - imgWidth + translateX + "px"
            });

            return;
          } // 默认居中


          _this.setState({
            top: imgTop,
            left: left + width / 2 - imgWidth / 2 + translateX + "px"
          });
        }
      };
    }
  };

  Guide.prototype.renderMask = function () {
    var _a = this.props,
        visible = _a.visible,
        loadablePointer = _a.loadablePointer,
        img = _a.img;
    var _b = this.state,
        top = _b.top,
        left = _b.left;
    var cls = this.prefixCls;
    var style = {
      left: left,
      top: top,
      display: visible ? 'block' : 'none'
    };

    if (visible || this.imgRef.current) {
      return React.createElement(Portal, null, React.createElement(React.Fragment, null, React.createElement(Mask, {
        className: cls + "-mask",
        visible: visible,
        timeout: 400,
        onClose: this.onClose
      }), loadablePointer ? React.createElement("img", {
        style: style,
        className: cls + "-icon",
        src: img,
        alt: "selection pointer",
        ref: this.imgRef
      }) : null));
    }

    return null;
  };

  Guide.prototype.render = function () {
    var cls = this.prefixCls;
    var _a = this.props,
        children = _a.children,
        visible = _a.visible;
    var className = visible ? cls + "-content" : null;

    if (typeof children === 'function') {
      return React.createElement(React.Fragment, null, children({
        className: className
      }), this.renderMask());
    }

    var child = React.Children.only(children);
    return React.createElement(React.Fragment, null, React.cloneElement(child, {
      className: classNames(className, child.props.className)
    }), this.renderMask());
  }; // 默认值


  Guide.defaultProps = {
    loadablePointer: false,
    img: UseConfig.assetCommonUiUrl + "/images/vehicle_pointer.png",
    position: 'center',
    translateX: 0,
    translateY: 0,
    expires: 3000
  };
  return Guide;
}(React.Component);

export default Guide;