var __assign = this && this.__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) {
        if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
      }
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

import React from 'react';
import { CSSTransition, TransitionGroup } from 'react-transition-group';
import classNames from 'classnames';
import ReactDOM from 'react-dom';
import { canUseDOM } from '../utils/string';
var defaultConfig = {
  expires: 3000,
  maxCount: 3,
  width: '6rem',
  height: 'auto',
  onDestroy: function onDestroy() {}
};
var prefixCls = 're-message';
var container = null;
var items = [];
var id = 0;
export default function Message(config) {
  if (!canUseDOM) {
    throw new Error('Message use client!');
  }

  var cachedId = id++;

  var currentConfig = __assign(__assign({}, defaultConfig), config);

  if (!container) {
    container = document.createElement('div');
    container.setAttribute('class', classNames(currentConfig.className, prefixCls + "-container"));
    document.body.appendChild(container);
  }

  if (items.length >= currentConfig.maxCount) {
    close(items[0].id);
  }

  function close(id) {
    var index = items.findIndex(function (item) {
      return item.id === id;
    });

    if (index >= 0) {
      clearTimeout(items[index].timer);
      items.splice(index, 1);
    } // 关闭需要再渲染一次


    render();
  }

  function _onExited(id) {
    currentConfig.onDestroy(id);
  } // 渲染组件


  function render() {
    var cls = prefixCls;
    ReactDOM.render(React.createElement(TransitionGroup, {
      component: null
    }, items.map(function (item) {
      var _a = item.currentConfig,
          content = _a.content,
          width = _a.width,
          height = _a.height,
          contentClassName = _a.contentClassName;
      var style = {
        width: width,
        height: height
      };
      return React.createElement(CSSTransition, {
        key: item.id,
        timeout: 300,
        classNames: cls + "-fade",
        onExited: function onExited() {
          _onExited(item.id);
        },
        appear: true
      }, React.createElement("div", {
        className: classNames(cls + "-content", contentClassName),
        style: style
      }, content, React.createElement("div", {
        className: cls + "-close",
        onClick: function onClick() {
          close(item.id);
        }
      }, "\xD7")));
    })), container);
  }

  var item = {
    currentConfig: currentConfig,
    id: cachedId,
    timer: null
  };
  items.push(item);
  render(); // render后设置自动关闭功能，把定时器绑定到列表

  item.timer = setTimeout(function () {
    close(cachedId);
  }, currentConfig.expires);

  function destroy() {
    items.forEach(function (item) {
      clearTimeout(item.timer);
    });
    items = [];
    render();
  }

  return destroy;
}