var __extends = this && this.__extends || function () {
  var _extendStatics = function extendStatics(d, b) {
    _extendStatics = Object.setPrototypeOf || {
      __proto__: []
    } instanceof Array && function (d, b) {
      d.__proto__ = b;
    } || function (d, b) {
      for (var p in b) {
        if (b.hasOwnProperty(p)) d[p] = b[p];
      }
    };

    return _extendStatics(d, b);
  };

  return function (d, b) {
    _extendStatics(d, b);

    function __() {
      this.constructor = d;
    }

    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
  };
}();

import React from 'react';
import { CSSTransition } from 'react-transition-group';
import classNames from 'classnames';
import Portal from '../portal';
import Title from './Title';
import Affix from './Affix';
import addEventListener from '../utils/addEventListener';
import { hasVerticalScrollbar, isTouchScreen } from '../utils/string';
import Mask from '../mask';
import { getElementSize, getStyle, getStyleValue } from '../utils/get';
import getPopupLocation from './getPopupLocation';
import { getScrollBarSize } from '../utils/getScrollBarSize';
import contains from '../utils/contains';

function getId() {
  try {
    return new Date().getTime();
  } catch (e) {
    return Math.random();
  }
}

var Modal =
/** @class */
function (_super) {
  __extends(Modal, _super);

  function Modal() {
    var _this = _super !== null && _super.apply(this, arguments) || this;

    _this.openId = getId();
    _this.prefixCls = 're-modal';
    _this.scrollBarSize = 0;
    _this.popupRef = React.createRef();
    _this.maskRef = React.createRef();
    /**
     * 全局事件回调
     * @param event
     */

    _this.onDocumentClick = function (event) {
      if (!_this.props.visible) {
        return;
      }

      var target = event.target;

      var popupEl = _this.getPopupDomNode();

      var maskEl = _this.getMaskDomNode();

      if (!target || !popupEl || !maskEl) {
        return;
      }

      var notCloseElement;

      if (_this.props.getNotCloseElement) {
        notCloseElement = _this.props.getNotCloseElement();
      } // 排除操作当前元素


      if (!contains(popupEl, target) && (maskEl === target || target === popupEl.parentNode) || notCloseElement && !contains(notCloseElement, target)) {
        _this.onClose(target, true);
      }
    };

    _this.clearPopupTimer = function (popupEl) {
      clearTimeout(_this.popupTimer);

      var element = popupEl || _this.getPopupDomNode();

      if (element) {
        element.style.visibility = '';
      }
    };

    _this.onClose = function (event, isOutsideClick) {
      if (isOutsideClick === void 0) {
        isOutsideClick = false;
      }

      var _a = _this.props,
          closable = _a.closable,
          maskClosable = _a.maskClosable,
          onClose = _a.onClose,
          name = _a.name;

      if ((isOutsideClick && maskClosable || !isOutsideClick && closable) && onClose) {
        onClose(event, name);
      }
    };

    _this.onEnter = function (popupEl) {
      var element = popupEl || _this.getPopupDomNode();

      if (!element) {
        return;
      }

      if (element && element.parentNode) {
        var parentNode = element.parentNode;
        parentNode.style.display = 'block';
      }

      _this.clearPopupTimer(element);

      var _a = _this.props,
          placement = _a.placement,
          hiddenScroll = _a.hiddenScroll,
          targetTop = _a.top,
          popupLimitSpacing = _a.popupLimitSpacing;

      if (hiddenScroll) {
        Modal.cacheOpenedIds.push(_this.openId);
        var currentDocument = window.document.body;
        currentDocument.style.overflow = 'hidden';

        if (hasVerticalScrollbar(_this.scrollBarSize)) {
          currentDocument.style.paddingRight = _this.scrollBarSize + "px";

          if (element && element.parentNode) {
            var parentNode = element.parentNode;
            parentNode.style.paddingRight = _this.scrollBarSize + "px";
          }
        }
      }

      var _b = getPopupLocation({
        placement: placement,
        element: element,
        targetTop: targetTop,
        popupLimitSpacing: popupLimitSpacing
      }),
          top = _b.top,
          height = _b.height;

      if (top) {
        element.style.top = top;
      }

      if (height) {
        element.style.height = height;
      }

      var isFixedTitle = _this.props.isFixedTitle; // 固定title

      if (isFixedTitle) {
        // 需要延迟执行
        _this.affixTimer = setTimeout(function () {
          element.style.overflow = 'hidden';
          var firstChild = element.firstChild;
          var lastChild = element.lastChild;
          var firstChildHeight = getStyleValue(firstChild, 'height');
          lastChild.style.overflow = 'auto';
          var popupHeight = height || _this.props.height;

          if (!popupHeight || popupHeight === 'auto') {
            lastChild.style.maxHeight = "calc(100vh - .3rem - " + (placement === 'top' && top ? top : '0px') + " - " + firstChildHeight + "px)";
          } else {
            lastChild.style.maxHeight = "calc(100% - " + firstChildHeight + "px)";
          }
        });
      }
    };

    _this.onEntered = function () {
      var _a = _this.props,
          onOpened = _a.onOpened,
          name = _a.name;

      if (onOpened) {
        onOpened(name);
      }
    };

    _this.onExited = function (element) {
      clearTimeout(_this.affixTimer);
      var _a = _this.props,
          hiddenScroll = _a.hiddenScroll,
          onDestroy = _a.onDestroy,
          name = _a.name;
      var currentDocument = window.document.body;

      if (hiddenScroll && currentDocument) {
        // 删除指定id
        Modal.cacheOpenedIds = Modal.cacheOpenedIds.filter(function (id) {
          return id !== _this.openId;
        }); // 如果都隐藏了，才清除body样式

        if (Modal.cacheOpenedIds.length === 0) {
          currentDocument.style.overflow = '';
          currentDocument.style.paddingRight = '';
        }
      }

      if (element && element.parentNode) {
        var parentNode = element.parentNode;
        parentNode.style.display = 'none';
        parentNode.style.paddingRight = '';
      }

      if (onDestroy) {
        onDestroy(name);
      }
    };

    return _this;
  }

  Modal.prototype.componentDidMount = function () {
    this.scrollBarSize = getScrollBarSize();
    this.componentDidUpdate();
  };

  Modal.prototype.getSnapshotBeforeUpdate = function () {
    var popupEl = this.getPopupDomNode();

    if (this.props.visible && popupEl) {
      var displayValue = getStyle(popupEl, 'display');

      if (displayValue === 'none') {
        return null;
      }

      return getElementSize(popupEl);
    }

    return null;
  };

  Modal.prototype.componentDidUpdate = function (_prevProps, _prevState, prevPopupSize) {
    var _this = this;

    if (!this.props.visible) {
      this.clearOutsideHandler();
      return;
    }

    var currentDocument;
    var touch = isTouchScreen();

    if (!this.clickOutsideHandler && !touch) {
      currentDocument = window.document;
      this.clickOutsideHandler = addEventListener(currentDocument, 'mousedown', this.onDocumentClick);
    } // always hide on mobile


    if (!this.touchOutsideHandler && touch) {
      currentDocument = currentDocument || window.document;
      this.touchOutsideHandler = addEventListener(currentDocument, 'touchstart', this.onDocumentClick);
    }

    if (!prevPopupSize) {
      return;
    }

    var popupEl = this.getPopupDomNode();

    if (popupEl && prevPopupSize) {
      var _a = getElementSize(popupEl),
          width = _a.width,
          height = _a.height;

      if (prevPopupSize.width !== width || prevPopupSize.height !== height) {
        this.clearPopupTimer(popupEl);
        popupEl.style.visibility = 'hidden';
        this.popupTimer = setTimeout(function () {
          _this.onEnter(popupEl);

          popupEl.style.visibility = '';
        });
      }
    }
  };

  Modal.prototype.componentWillUnmount = function () {
    // remove绑定事件
    this.clearOutsideHandler();
    this.onExited();
    clearTimeout(this.affixTimer);
  };

  Modal.prototype.clearOutsideHandler = function () {
    this.clearPopupTimer();

    if (this.clickOutsideHandler) {
      this.clickOutsideHandler.remove();
      this.clickOutsideHandler = null;
    }

    if (this.touchOutsideHandler) {
      this.touchOutsideHandler.remove();
      this.touchOutsideHandler = null;
    }
  };

  Modal.prototype.getPopupDomNode = function () {
    if (this.popupRef && this.popupRef.current) {
      return this.popupRef.current;
    }

    return null;
  };

  Modal.prototype.getMaskDomNode = function () {
    if (this.maskRef && this.maskRef.current) {
      return this.maskRef.current;
    }

    return null;
  };

  Modal.prototype.renderPopup = function () {
    var _a;

    var cls = this.prefixCls;
    var _b = this.props,
        visible = _b.visible,
        children = _b.children,
        width = _b.width,
        height = _b.height,
        className = _b.className,
        closable = _b.closable,
        title = _b.title,
        placement = _b.placement,
        top = _b.top;
    var maxHeight = "calc(100vh - .3rem - " + (placement === 'top' && top ? top : '0px') + ")";
    return React.createElement("div", {
      className: classNames(cls, className, (_a = {}, _a[cls + "-center"] = placement === 'center', _a))
    }, React.createElement(CSSTransition, {
      in: visible,
      timeout: 300,
      classNames: cls + "-fade",
      onEnter: this.onEnter,
      onEntered: this.onEntered,
      onExited: this.onExited,
      appear: true
    }, React.createElement("div", {
      className: cls + "-content",
      style: {
        width: width,
        height: height,
        maxHeight: maxHeight
      },
      ref: this.popupRef
    }, React.createElement(Title, {
      prefixCls: cls,
      closable: closable,
      onClose: this.onClose,
      title: title
    }), children)));
  };

  Modal.prototype.render = function () {
    var visible = this.props.visible;

    if (visible || this.popupRef.current) {
      return React.createElement(Portal, null, React.createElement(React.Fragment, null, React.createElement(Mask, {
        visible: visible,
        timeout: 500,
        ref: this.maskRef
      }), this.renderPopup()));
    }

    return null;
  };

  Modal.Affix = Affix;
  Modal.cacheOpenedIds = []; // 默认值

  Modal.defaultProps = {
    placement: 'center',
    width: '6.5rem',
    height: 'auto',
    top: '10%',
    popupLimitSpacing: 14,
    closable: true,
    maskClosable: true,
    hiddenScroll: true
  };
  return Modal;
}(React.Component);

export default Modal;