function getAvailPosition(rootInfo, isInsertNode) {
  return isInsertNode ? {
    left: rootInfo.offsetLeft,
    top: rootInfo.offsetTop
  } : rootInfo;
}

export function getAvailSize(ignoreScroll) {
  if (document.compatMode === 'BackCompat') {
    if (ignoreScroll) {
      return {
        width: document.body.clientWidth,
        height: document.body.clientHeight
      };
    }

    return {
      width: document.body.scrollWidth,
      height: document.body.scrollHeight
    };
  }

  if (ignoreScroll) {
    return {
      width: document.documentElement.clientWidth,
      height: document.documentElement.clientHeight
    };
  }

  return {
    // 这里使用min, 原因在于当撑出窗口宽度时，计算的值是width + scrollLeft, 此时再做不让其左右滚动，就会出现计算错误
    width: Math.min(document.documentElement.scrollWidth, document.documentElement.clientWidth),
    height: Math.max(document.documentElement.scrollHeight, document.documentElement.clientHeight)
  };
}
/* 具体方位 */

export function getLeftLocationInfo(_a) {
  var rootInfo = _a.rootInfo,
      popInfo = _a.popInfo,
      _b = _a.translateY,
      translateY = _b === void 0 ? 0 : _b,
      rootToPopupSpacing = _a.rootToPopupSpacing,
      isTransformHorizontalDirection = _a.isTransformHorizontalDirection,
      hiddenArrow = _a.hiddenArrow,
      scrollTop = _a.scrollTop,
      scrollLeft = _a.scrollLeft,
      isInsertNode = _a.isInsertNode;
  var arrowTop = popInfo.height / 2 - translateY;

  var _c = getAvailPosition(rootInfo, isInsertNode),
      left = _c.left,
      top = _c.top;

  return {
    rootWidth: rootInfo.width,
    width: popInfo.width,
    height: popInfo.height,
    left: left - (rootToPopupSpacing + popInfo.width) + scrollLeft,
    top: top - (popInfo.height / 2 - rootInfo.height / 2) + translateY + scrollTop,
    arrowStyle: hiddenArrow ? {} : {
      top: arrowTop + "px",
      right: '-4px',
      transform: 'translateY(-50%) rotate(-45deg)'
    },
    transformOrigin: isTransformHorizontalDirection ? 'right center' : "right " + arrowTop + "px"
  };
}
export function getRightLocationInfo(_a) {
  var rootInfo = _a.rootInfo,
      popInfo = _a.popInfo,
      _b = _a.translateY,
      translateY = _b === void 0 ? 0 : _b,
      rootToPopupSpacing = _a.rootToPopupSpacing,
      isTransformHorizontalDirection = _a.isTransformHorizontalDirection,
      hiddenArrow = _a.hiddenArrow,
      scrollTop = _a.scrollTop,
      scrollLeft = _a.scrollLeft,
      isInsertNode = _a.isInsertNode;
  var arrowTop = popInfo.height / 2 - translateY;

  var _c = getAvailPosition(rootInfo, isInsertNode),
      left = _c.left,
      top = _c.top;

  return {
    rootWidth: rootInfo.width,
    width: popInfo.width,
    height: popInfo.height,
    left: left + rootToPopupSpacing + rootInfo.width + scrollLeft,
    top: top - (popInfo.height / 2 - rootInfo.height / 2) + translateY + scrollTop,
    arrowStyle: hiddenArrow ? {} : {
      top: arrowTop + "px",
      left: '-4px',
      transform: 'translateY(-50%) rotate(-225deg)'
    },
    transformOrigin: isTransformHorizontalDirection ? 'left center' : "left " + arrowTop + "px"
  };
}
export function getTopLocationInfo(_a) {
  var rootInfo = _a.rootInfo,
      popInfo = _a.popInfo,
      _b = _a.translateX,
      translateX = _b === void 0 ? 0 : _b,
      rootToPopupSpacing = _a.rootToPopupSpacing,
      isTransformHorizontalDirection = _a.isTransformHorizontalDirection,
      hiddenArrow = _a.hiddenArrow,
      scrollTop = _a.scrollTop,
      scrollLeft = _a.scrollLeft,
      isInsertNode = _a.isInsertNode;
  var arrowLeft = popInfo.width / 2 - translateX;

  var _c = getAvailPosition(rootInfo, isInsertNode),
      left = _c.left,
      top = _c.top;

  return {
    rootWidth: rootInfo.width,
    width: popInfo.width,
    height: popInfo.height,
    left: left + rootInfo.width / 2 - popInfo.width / 2 + translateX + scrollLeft,
    top: top - (rootToPopupSpacing + popInfo.height) + scrollTop,
    arrowStyle: hiddenArrow ? {} : {
      left: arrowLeft + "px",
      bottom: '-4px',
      transform: 'translateX(-50%) rotate(45deg)'
    },
    transformOrigin: isTransformHorizontalDirection ? 'center bottom' : arrowLeft + "px bottom"
  };
}
export function getBottomLocationInfo(_a) {
  var rootInfo = _a.rootInfo,
      popInfo = _a.popInfo,
      _b = _a.translateX,
      translateX = _b === void 0 ? 0 : _b,
      rootToPopupSpacing = _a.rootToPopupSpacing,
      isTransformHorizontalDirection = _a.isTransformHorizontalDirection,
      hiddenArrow = _a.hiddenArrow,
      scrollTop = _a.scrollTop,
      scrollLeft = _a.scrollLeft,
      isInsertNode = _a.isInsertNode;
  var arrowLeft = popInfo.width / 2 - translateX;

  var _c = getAvailPosition(rootInfo, isInsertNode),
      left = _c.left,
      top = _c.top;

  return {
    rootWidth: rootInfo.width,
    width: popInfo.width,
    height: popInfo.height,
    left: left + rootInfo.width / 2 - popInfo.width / 2 + translateX + scrollLeft,
    top: top + rootInfo.height + rootToPopupSpacing + scrollTop,
    arrowStyle: hiddenArrow ? {} : {
      left: arrowLeft + "px",
      top: '-4px',
      transform: 'translateX(-50%) rotate(-135deg)'
    },
    transformOrigin: isTransformHorizontalDirection ? 'center top' : arrowLeft + "px top"
  };
}
/* 其他方位 */

export function getLeftTopLocationInfo(_a) {
  var rootInfo = _a.rootInfo,
      popInfo = _a.popInfo,
      _b = _a.translateY,
      translateY = _b === void 0 ? 0 : _b,
      rootToPopupSpacing = _a.rootToPopupSpacing,
      isTransformHorizontalDirection = _a.isTransformHorizontalDirection,
      hiddenArrow = _a.hiddenArrow,
      scrollTop = _a.scrollTop,
      scrollLeft = _a.scrollLeft,
      isInsertNode = _a.isInsertNode;
  var arrowTop = rootInfo.height / 2 - translateY;

  var _c = getAvailPosition(rootInfo, isInsertNode),
      left = _c.left,
      top = _c.top;

  return {
    rootWidth: rootInfo.width,
    width: popInfo.width,
    height: popInfo.height,
    left: left - (rootToPopupSpacing + popInfo.width) + scrollLeft,
    top: top + translateY + scrollTop,
    arrowStyle: hiddenArrow ? {} : {
      top: arrowTop + "px",
      right: '-4px',
      transform: 'translateY(-50%) rotate(-45deg)'
    },
    transformOrigin: isTransformHorizontalDirection ? 'right center' : "right " + arrowTop + "px"
  };
}
export function getLeftBottomLocationInfo(_a) {
  var rootInfo = _a.rootInfo,
      popInfo = _a.popInfo,
      _b = _a.translateY,
      translateY = _b === void 0 ? 0 : _b,
      rootToPopupSpacing = _a.rootToPopupSpacing,
      isTransformHorizontalDirection = _a.isTransformHorizontalDirection,
      hiddenArrow = _a.hiddenArrow,
      scrollTop = _a.scrollTop,
      scrollLeft = _a.scrollLeft,
      isInsertNode = _a.isInsertNode;
  var arrowTop = popInfo.height - rootInfo.height / 2 - translateY;

  var _c = getAvailPosition(rootInfo, isInsertNode),
      left = _c.left,
      top = _c.top;

  return {
    rootWidth: rootInfo.width,
    width: popInfo.width,
    height: popInfo.height,
    left: left - (rootToPopupSpacing + popInfo.width) + scrollLeft,
    top: top + rootInfo.height - popInfo.height + translateY + scrollTop,
    arrowStyle: hiddenArrow ? {} : {
      top: arrowTop + "px",
      right: '-4px',
      transform: 'translateY(-50%) rotate(-45deg)'
    },
    transformOrigin: isTransformHorizontalDirection ? 'right center' : "right " + arrowTop + "px"
  };
}
export function getRightTopLocationInfo(_a) {
  var rootInfo = _a.rootInfo,
      popInfo = _a.popInfo,
      _b = _a.translateY,
      translateY = _b === void 0 ? 0 : _b,
      rootToPopupSpacing = _a.rootToPopupSpacing,
      isTransformHorizontalDirection = _a.isTransformHorizontalDirection,
      hiddenArrow = _a.hiddenArrow,
      scrollTop = _a.scrollTop,
      scrollLeft = _a.scrollLeft,
      isInsertNode = _a.isInsertNode;
  var arrowTop = rootInfo.height / 2 - translateY;

  var _c = getAvailPosition(rootInfo, isInsertNode),
      left = _c.left,
      top = _c.top;

  return {
    rootWidth: rootInfo.width,
    width: popInfo.width,
    height: popInfo.height,
    left: left + rootToPopupSpacing + rootInfo.width + scrollLeft,
    top: top + translateY + scrollTop,
    arrowStyle: hiddenArrow ? {} : {
      top: arrowTop + "px",
      left: '-4px',
      transform: 'translateY(-50%) rotate(-225deg)'
    },
    transformOrigin: isTransformHorizontalDirection ? 'left center' : "left " + arrowTop + "px"
  };
}
export function getRightBottomLocationInfo(_a) {
  var rootInfo = _a.rootInfo,
      popInfo = _a.popInfo,
      _b = _a.translateY,
      translateY = _b === void 0 ? 0 : _b,
      rootToPopupSpacing = _a.rootToPopupSpacing,
      isTransformHorizontalDirection = _a.isTransformHorizontalDirection,
      hiddenArrow = _a.hiddenArrow,
      scrollTop = _a.scrollTop,
      scrollLeft = _a.scrollLeft,
      isInsertNode = _a.isInsertNode;
  var arrowTop = popInfo.height - rootInfo.height / 2 - translateY;

  var _c = getAvailPosition(rootInfo, isInsertNode),
      left = _c.left,
      top = _c.top;

  return {
    rootWidth: rootInfo.width,
    width: popInfo.width,
    height: popInfo.height,
    left: left + rootToPopupSpacing + rootInfo.width + scrollLeft,
    top: top + rootInfo.height - popInfo.height + translateY + scrollTop,
    arrowStyle: hiddenArrow ? {} : {
      top: arrowTop + "px",
      left: '-4px',
      transform: 'translateY(-50%) rotate(-225deg)'
    },
    transformOrigin: isTransformHorizontalDirection ? 'left center' : "left " + arrowTop + "px"
  };
}
export function getTopLeftLocationInfo(_a) {
  var rootInfo = _a.rootInfo,
      popInfo = _a.popInfo,
      _b = _a.translateX,
      translateX = _b === void 0 ? 0 : _b,
      rootToPopupSpacing = _a.rootToPopupSpacing,
      isTransformHorizontalDirection = _a.isTransformHorizontalDirection,
      hiddenArrow = _a.hiddenArrow,
      scrollTop = _a.scrollTop,
      scrollLeft = _a.scrollLeft,
      isInsertNode = _a.isInsertNode;
  var arrowLeft = rootInfo.width / 2 - translateX;

  var _c = getAvailPosition(rootInfo, isInsertNode),
      left = _c.left,
      top = _c.top;

  return {
    rootWidth: rootInfo.width,
    width: popInfo.width,
    height: popInfo.height,
    left: left + translateX + scrollLeft,
    top: top - (rootToPopupSpacing + popInfo.height) + scrollTop,
    arrowStyle: hiddenArrow ? {} : {
      left: arrowLeft + "px",
      bottom: '-4px',
      transform: 'translateX(-50%) rotate(45deg)'
    },
    transformOrigin: isTransformHorizontalDirection ? 'center bottom' : arrowLeft + "px bottom"
  };
}
export function getTopRightLocationInfo(_a) {
  var rootInfo = _a.rootInfo,
      popInfo = _a.popInfo,
      _b = _a.translateX,
      translateX = _b === void 0 ? 0 : _b,
      rootToPopupSpacing = _a.rootToPopupSpacing,
      isTransformHorizontalDirection = _a.isTransformHorizontalDirection,
      hiddenArrow = _a.hiddenArrow,
      scrollTop = _a.scrollTop,
      scrollLeft = _a.scrollLeft,
      isInsertNode = _a.isInsertNode;
  var arrowLeft = popInfo.width - rootInfo.width / 2 - translateX;

  var _c = getAvailPosition(rootInfo, isInsertNode),
      left = _c.left,
      top = _c.top;

  return {
    rootWidth: rootInfo.width,
    width: popInfo.width,
    height: popInfo.height,
    left: left - (popInfo.width - rootInfo.width) + translateX + scrollLeft,
    top: top - (rootToPopupSpacing + popInfo.height) + scrollTop,
    arrowStyle: hiddenArrow ? {} : {
      left: arrowLeft + "px",
      bottom: '-4px',
      transform: 'translateX(-50%) rotate(45deg)'
    },
    transformOrigin: isTransformHorizontalDirection ? 'center bottom' : arrowLeft + "px bottom"
  };
}
export function getBottomLeftLocationInfo(_a) {
  var rootInfo = _a.rootInfo,
      popInfo = _a.popInfo,
      _b = _a.translateX,
      translateX = _b === void 0 ? 0 : _b,
      rootToPopupSpacing = _a.rootToPopupSpacing,
      isTransformHorizontalDirection = _a.isTransformHorizontalDirection,
      hiddenArrow = _a.hiddenArrow,
      scrollTop = _a.scrollTop,
      scrollLeft = _a.scrollLeft,
      isInsertNode = _a.isInsertNode;
  var arrowLeft = rootInfo.width / 2 - translateX;

  var _c = getAvailPosition(rootInfo, isInsertNode),
      left = _c.left,
      top = _c.top;

  return {
    rootWidth: rootInfo.width,
    width: popInfo.width,
    height: popInfo.height,
    left: left + translateX + scrollLeft,
    top: top + rootInfo.height + rootToPopupSpacing + scrollTop,
    arrowStyle: hiddenArrow ? {} : {
      left: arrowLeft + "px",
      top: '-4px',
      transform: 'translateX(-50%) rotate(-135deg)'
    },
    transformOrigin: isTransformHorizontalDirection ? 'center top' : arrowLeft + "px top"
  };
}
export function getBottomRightLocationInfo(_a) {
  var rootInfo = _a.rootInfo,
      popInfo = _a.popInfo,
      _b = _a.translateX,
      translateX = _b === void 0 ? 0 : _b,
      rootToPopupSpacing = _a.rootToPopupSpacing,
      isTransformHorizontalDirection = _a.isTransformHorizontalDirection,
      hiddenArrow = _a.hiddenArrow,
      scrollTop = _a.scrollTop,
      scrollLeft = _a.scrollLeft,
      isInsertNode = _a.isInsertNode;
  var arrowLeft = popInfo.width - rootInfo.width / 2 - translateX;

  var _c = getAvailPosition(rootInfo, isInsertNode),
      left = _c.left,
      top = _c.top;

  return {
    rootWidth: rootInfo.width,
    width: popInfo.width,
    height: popInfo.height,
    left: left - (popInfo.width - rootInfo.width) + translateX + scrollLeft,
    top: top + rootInfo.height + rootToPopupSpacing + scrollTop,
    arrowStyle: hiddenArrow ? {} : {
      left: arrowLeft + "px",
      top: '-4px',
      transform: 'translateX(-50%) rotate(-135deg)'
    },
    transformOrigin: isTransformHorizontalDirection ? 'center top' : arrowLeft + "px top"
  };
}