var __assign = this && this.__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) {
        if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
      }
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

import { LocationType } from './constants';
import { getOffset, getScrollValue, getStyle } from '../utils/get';
import { getAvailSize, getTopLocationInfo, getBottomLocationInfo, getLeftLocationInfo, getRightLocationInfo, getLeftTopLocationInfo, getLeftBottomLocationInfo, getRightTopLocationInfo, getRightBottomLocationInfo, getTopLeftLocationInfo, getTopRightLocationInfo, getBottomLeftLocationInfo, getBottomRightLocationInfo } from './align';

function getTranslateX(_a, availSize, initTranslateX, popupLimitSpacing, scrollValue) {
  var left = _a.left,
      width = _a.width;
  var limit = initTranslateX ? 0 : popupLimitSpacing;

  if (width < availSize.width - limit * 2) {
    if (left - scrollValue.left < limit) {
      return {
        translateX: -left + scrollValue.left + limit,
        maxWidth: undefined
      };
    }

    if (left - scrollValue.left + width + limit > availSize.width) {
      return {
        translateX: availSize.width - (left + width + limit) + scrollValue.left,
        maxWidth: undefined
      };
    }
  } else {
    var maxWidth = availSize.width - limit * 2;
    return {
      translateX: availSize.width - (left + maxWidth + limit) + scrollValue.left,
      maxWidth: maxWidth
    };
  }

  return {
    translateX: initTranslateX,
    maxWidth: undefined
  };
}

function getTranslateY(_a, availSize, popupLimitSpacing, scrollValue) {
  var top = _a.top,
      height = _a.height;

  if (top - scrollValue.top < popupLimitSpacing) {
    return -top + scrollValue.top + popupLimitSpacing;
  }

  if (height < availSize.height && top - scrollValue.top + height + popupLimitSpacing > availSize.height) {
    return availSize.height - top + scrollValue.top - height - popupLimitSpacing;
  }

  return 0;
}
/**
 * 获取 popup 元素接下来的位置信息
 * @returns {*}
 */


export default function getPopupLocation(placement, rootEl, popupEl, initTranslateX, initTranslateY, rootToPopupSpacing, popupLimitSpacing, isTransformHorizontalDirection, hiddenArrow, isInsertNode) {
  var isHidden = false; // 判断body是否有隐藏, 并计算可用区域大小

  try {
    var bodyOverflowValue = getStyle(document.body, 'overflow');
    isHidden = bodyOverflowValue === 'hidden';
  } catch (e) {}

  var rootInfo = getOffset(rootEl, isHidden);
  var popInfo = getOffset(popupEl, isHidden);
  var availSize = getAvailSize(isHidden);
  var parentNodeInfo = {
    left: 0,
    top: 0
  };
  var leftPopupLimitSpacing = popupLimitSpacing;
  var topPopupLimitSpacing = popupLimitSpacing; // 悬浮框插入到文档流中

  if (isInsertNode) {
    var parentNode = rootEl.parentNode; // 判断父级元素是否定位，否则添加relative

    if (getStyle(parentNode, 'position') === 'static') {
      parentNode.style.position = 'relative';
    }

    parentNodeInfo = getOffset(parentNode); // 扩充popup临界点

    leftPopupLimitSpacing = -parentNodeInfo.left + leftPopupLimitSpacing;
    topPopupLimitSpacing = -parentNodeInfo.top + topPopupLimitSpacing;
  }

  var scrollValue = {
    left: 0,
    top: 0
  };

  if (isHidden) {
    scrollValue = getScrollValue();
  }

  var params = {
    rootInfo: rootInfo,
    popInfo: popInfo,
    isTransformHorizontalDirection: isTransformHorizontalDirection,
    rootToPopupSpacing: rootToPopupSpacing,
    hiddenArrow: hiddenArrow,
    translateX: initTranslateX,
    translateY: initTranslateY,
    scrollTop: scrollValue.top,
    scrollLeft: scrollValue.left,
    isInsertNode: isInsertNode
  };

  if (placement === LocationType.Top.value) {
    var result = getTopLocationInfo(params);
    var translateX = getTranslateX(__assign(__assign({}, result), {
      left: isInsertNode ? parentNodeInfo.left + result.left : result.left
    }), availSize, initTranslateX, popupLimitSpacing, scrollValue).translateX;

    if (result.top - scrollValue.top < topPopupLimitSpacing) {
      return getBottomLocationInfo(__assign(__assign({}, params), {
        translateX: translateX
      }));
    }

    if (!translateX) {
      return result;
    }

    return getTopLocationInfo(__assign(__assign({}, params), {
      translateX: translateX
    }));
  }

  if (placement === LocationType.Bottom.value) {
    var result = getBottomLocationInfo(params);
    var translateX = getTranslateX(__assign(__assign({}, result), {
      left: isInsertNode ? parentNodeInfo.left + result.left : result.left
    }), availSize, initTranslateX, popupLimitSpacing, scrollValue).translateX; // 已经超出底部

    if ( // 判断底部是否能装下悬浮框
    (isInsertNode && result.top - scrollValue.top + popInfo.height - topPopupLimitSpacing > availSize.height || result.top - scrollValue.top + result.height + topPopupLimitSpacing > availSize.height) && // 判断rootElement上半部分是否能装下悬浮框
    rootInfo.top - (rootToPopupSpacing + popInfo.height) > topPopupLimitSpacing) {
      return getTopLocationInfo(__assign(__assign({}, params), {
        translateX: translateX
      }));
    } // 正常情况直接返回


    if (!translateX) {
      return result;
    }

    return getBottomLocationInfo(__assign(__assign({}, params), {
      translateX: translateX
    }));
  }

  if (placement === LocationType.Left.value) {
    var result = getLeftLocationInfo(params);
    var translateY = getTranslateY(result, availSize, topPopupLimitSpacing, scrollValue);

    if (result.left - scrollValue.left < leftPopupLimitSpacing) {
      return getRightLocationInfo(__assign(__assign({}, params), {
        translateY: translateY
      }));
    }

    if (!translateY) {
      return result;
    }

    return getLeftLocationInfo(__assign(__assign({}, params), {
      translateY: translateY
    }));
  }

  if (placement === LocationType.Right.value) {
    var result = getRightLocationInfo(params);
    var translateY = getTranslateY(result, availSize, topPopupLimitSpacing, scrollValue);

    if ( // 判断右半部是否能装下悬浮框
    (isInsertNode && result.left - scrollValue.left + popInfo.width - leftPopupLimitSpacing > availSize.width || result.left - scrollValue.left + result.width + leftPopupLimitSpacing > availSize.width) && // 判断rootElement左半部分是否能装下悬浮框
    rootInfo.left - (rootToPopupSpacing + popInfo.width) > leftPopupLimitSpacing) {
      return getLeftLocationInfo(__assign(__assign({}, params), {
        translateY: translateY
      }));
    } // 不再重新计算


    if (!translateY) {
      return result;
    }

    return getRightLocationInfo(__assign(__assign({}, params), {
      translateY: translateY
    }));
  }

  if (placement === LocationType.LeftTop.value || placement === LocationType.LeftBottom.value) {
    var isLeftTop = placement === LocationType.LeftTop.value;
    var result = isLeftTop ? getLeftTopLocationInfo(params) : getLeftBottomLocationInfo(params);
    var translateY = getTranslateY(result, availSize, topPopupLimitSpacing, scrollValue);

    if (result.left - scrollValue.left < leftPopupLimitSpacing) {
      return isLeftTop ? getRightTopLocationInfo(__assign(__assign({}, params), {
        translateY: translateY
      })) : getRightBottomLocationInfo(__assign(__assign({}, params), {
        translateY: translateY
      }));
    }

    if (!translateY) {
      return result;
    }

    return isLeftTop ? getLeftTopLocationInfo(__assign(__assign({}, params), {
      translateY: translateY
    })) : getLeftBottomLocationInfo(__assign(__assign({}, params), {
      translateY: translateY
    }));
  }

  if (placement === LocationType.RightTop.value || placement === LocationType.RightBottom.value) {
    var isRightTop = placement === LocationType.RightTop.value;
    var result = isRightTop ? getRightTopLocationInfo(params) : getRightBottomLocationInfo(params);
    var translateY = getTranslateY(result, availSize, topPopupLimitSpacing, scrollValue);

    if ( // 判断右半部是否能装下悬浮框
    (isInsertNode && result.left - scrollValue.left + popInfo.width - leftPopupLimitSpacing > availSize.width || result.left - scrollValue.left + result.width + leftPopupLimitSpacing > availSize.width) && // 判断rootElement左半部分是否能装下悬浮框
    rootInfo.left - (rootToPopupSpacing + popInfo.width) > leftPopupLimitSpacing) {
      return isRightTop ? getLeftTopLocationInfo(__assign(__assign({}, params), {
        translateY: translateY
      })) : getLeftBottomLocationInfo(__assign(__assign({}, params), {
        translateY: translateY
      }));
    }

    if (!translateY) {
      return result;
    }

    return isRightTop ? getRightTopLocationInfo(__assign(__assign({}, params), {
      translateY: translateY
    })) : getRightBottomLocationInfo(__assign(__assign({}, params), {
      translateY: translateY
    }));
  }

  if (placement === LocationType.TopLeft.value || placement === LocationType.TopRight.value) {
    var isTopLeft = placement === LocationType.TopLeft.value;
    var result = isTopLeft ? getTopLeftLocationInfo(params) : getTopRightLocationInfo(params);
    var translateX = getTranslateX(__assign(__assign({}, result), {
      left: isInsertNode ? parentNodeInfo.left + result.left : result.left
    }), availSize, initTranslateX, popupLimitSpacing, scrollValue).translateX;

    if (result.top - scrollValue.top < topPopupLimitSpacing) {
      return isTopLeft ? getBottomLeftLocationInfo(__assign(__assign({}, params), {
        translateX: translateX
      })) : getBottomRightLocationInfo(__assign(__assign({}, params), {
        translateX: translateX
      }));
    }

    if (!translateX) {
      return result;
    }

    return isTopLeft ? getTopLeftLocationInfo(__assign(__assign({}, params), {
      translateX: translateX
    })) : getTopRightLocationInfo(__assign(__assign({}, params), {
      translateX: translateX
    }));
  }

  if (placement === LocationType.BottomLeft.value || placement === LocationType.BottomRight.value) {
    var isBottomLeft = placement === LocationType.BottomLeft.value;
    var result = isBottomLeft ? getBottomLeftLocationInfo(params) : getBottomRightLocationInfo(params);

    var _a = getTranslateX(__assign(__assign({}, result), {
      left: isInsertNode ? parentNodeInfo.left + result.left : result.left
    }), availSize, initTranslateX, popupLimitSpacing, scrollValue),
        translateX = _a.translateX,
        maxWidth = _a.maxWidth;

    if ( // 判断底部是否能装下悬浮框
    (isInsertNode && result.top - scrollValue.top + popInfo.height - topPopupLimitSpacing > availSize.height || result.top - scrollValue.top + result.height + topPopupLimitSpacing > availSize.height) && // 判断rootElement上半部分是否能装下悬浮框
    rootInfo.top - (rootToPopupSpacing + popInfo.height) > topPopupLimitSpacing) {
      return isBottomLeft ? getTopLeftLocationInfo(__assign(__assign({}, params), {
        translateX: translateX
      })) : getTopRightLocationInfo(__assign(__assign({}, params), {
        translateX: translateX
      }));
    } // 正常情况直接返回


    if (!translateX) {
      return result;
    }

    var value = isBottomLeft ? getBottomLeftLocationInfo(__assign(__assign({}, params), {
      translateX: translateX
    })) : getBottomRightLocationInfo(__assign(__assign({}, params), {
      translateX: translateX
    })); // 限制宽度

    value.maxWidth = maxWidth;
    return value;
  }

  throw new TypeError('Support placement: Top, TopLeft TopRight, Bottom, BottomLeft, BottomRight, Right, RightTop, RightBottom, Left, LeftTop, LeftBottom');
}