declare const baseConfig: {
    cache: string;
    credentials: string;
    referrerPolicy: string;
    headers: {
        'If-Modified-Since': string;
        'content-type': string;
    };
    mode: string;
    redirect: string;
};
declare type BaseConfig = typeof baseConfig;
export declare type RetryCondition = (res: any) => boolean;
export interface FetchOptions extends Partial<BaseConfig> {
    headers: any;
}
export declare type ResponseUse<T> = (isSuccess: boolean, res: T, options?: FetchOptions, url?: string, extraInfo?: {
    startTime?: number;
    retryCount?: number;
}) => T;
export declare type RequestUse<T> = (options: T, surplusParams?: object, abortController?: AbortController) => T | boolean;
export declare type CacheInfo = {
    isCache: boolean;
    cacheKey: string[] | string;
};
export declare type SetCacheInfo = (url: any, options: any, surplusParams: any) => CacheInfo;
export declare type CacheMethod = {
    getData: (key: string, namespace: string) => any;
    setData: (key: string, namespace: string, data: any) => any;
};
export interface Options extends FetchOptions {
    baseUrl?: string;
    requestUse?: RequestUse<any>;
    responseUse?: ResponseUse<any>;
    retryCount?: number;
    retryCondition?: RetryCondition;
    cacheMethod?: CacheMethod;
    setCacheInfo?: SetCacheInfo;
    cacheNamespace?: string;
}
export default class Config {
    static baseUrl?: string;
    static retryCount?: number;
    static retryCondition?: RetryCondition;
    static fetchOptions: FetchOptions;
    static requestUse?: RequestUse<any>;
    static cacheMethod?: CacheMethod;
    static cacheNamespace?: string;
    static responseUse?: ResponseUse<any>;
    static setOptions(options: Options): void;
}
export {};
