"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = Confirm;

var _react = _interopRequireDefault(require("react"));

var _reactDom = _interopRequireDefault(require("react-dom"));

var _classnames = _interopRequireDefault(require("classnames"));

var _string = require("../utils/string");

var _modal = _interopRequireDefault(require("../modal"));

var __assign = void 0 && (void 0).__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) {
        if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
      }
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

var prefixCls = 're-confirm';
var defaultConfig = {
  hiddenCancel: false,
  hiddenOk: false,
  cancelText: 'Cancel',
  cancelClassName: prefixCls + "-cancel",
  okText: 'Confirm',
  okClassName: prefixCls + "-ok",
  content: 'Are you sure?',
  width: '3.6rem',
  placement: 'center',
  visible: true,
  onOk: function onOk() {},
  onCancel: function onCancel() {}
};

function Confirm(config) {
  if (!_string.canUseDOM) {
    throw new Error('Confirm use client!');
  }

  var div = document.createElement('div');
  document.body.appendChild(div);
  var currentConfig = defaultConfig;

  function updateCurrentConfig(config) {
    var cancelClassName = (0, _classnames.default)(currentConfig.cancelClassName, config.cancelClassName);
    var okClassName = (0, _classnames.default)(currentConfig.okClassName, config.okClassName);
    currentConfig = __assign(__assign(__assign({}, currentConfig), config), {
      cancelClassName: cancelClassName,
      okClassName: okClassName,
      onDestroy: onDestroy
    });
  }

  updateCurrentConfig(config);

  function close(type) {
    if (type === void 0) {
      type = 'cancel';
    }

    currentConfig = __assign(__assign({}, currentConfig), {
      visible: false
    });
    render(currentConfig);

    if (type === 'ok') {
      currentConfig.onOk();
    } else {
      currentConfig.onCancel();
    }
  }

  function onDestroy() {
    var unmountResult = _reactDom.default.unmountComponentAtNode(div);

    if (unmountResult && div.parentNode) {
      try {
        div.parentNode.removeChild(div);
      } catch (e) {}
    }
  }

  function onVisible() {
    close('cancel');
  }

  function onConfirm() {
    close('ok');
  }

  function render(props) {
    _reactDom.default.render(_react.default.createElement(_modal.default, __assign({}, props, {
      className: (0, _classnames.default)(prefixCls + "-wrap", props.className),
      visible: props.visible,
      onClose: onVisible
    }), props.content, props.hiddenCancel && props.hiddenOk ? null : _react.default.createElement("div", {
      className: prefixCls + "-btns"
    }, props.hiddenCancel ? null : _react.default.createElement("button", {
      className: props.cancelClassName,
      onClick: onVisible
    }, props.cancelText), props.hiddenOk ? null : _react.default.createElement("button", {
      className: props.okClassName,
      onClick: onConfirm
    }, props.okText))), div);
  } // 渲染组件


  render(currentConfig);
  return {
    destroy: close,
    forceUpdate: function forceUpdate(config) {
      updateCurrentConfig(config);
      render(currentConfig);
    }
  };
}