"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _classnames = _interopRequireDefault(require("classnames"));

var _reactDom = _interopRequireDefault(require("react-dom"));

var _useConfig = _interopRequireDefault(require("../use-config"));

var _string = require("../utils/string");

var _loadingIcon = _interopRequireDefault(require("../loading-icon"));

var __assign = void 0 && (void 0).__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) {
        if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
      }
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

// 缓存创建的loading元素
var loadingElement = null;
var loadingTimer = null;
var closeTimer = null;
var cls = 're-loading';

function getLoadingContainer(isRectLoading) {
  var _a;

  return _react.default.createElement("div", {
    className: (0, _classnames.default)(cls + "-wrap", (_a = {}, _a[cls + "-rect-loading"] = isRectLoading, _a))
  }, isRectLoading ? _react.default.createElement("img", {
    className: cls + "-content",
    src: _useConfig.default.assetCommonUiUrl + "/images/p_loading_rect.gif",
    alt: "loading..."
  }) : _react.default.createElement(_loadingIcon.default, {
    className: cls + "-content"
  }));
} // openIds 容器


var openIds = []; // closeIds 容器

var closeIds = [];
var prevIsRectLoading = false;

function createLoading(isRectLoading) {
  loadingElement = document.createElement('div');
  document.body.appendChild(loadingElement);

  _reactDom.default.render(getLoadingContainer(isRectLoading), loadingElement);
}

var Loading =
/** @class */
function () {
  function Loading() {}

  Loading.open = function (config) {
    if (!_string.canUseDOM) {
      return null;
    }

    config = __assign({
      isRectLoading: false,
      expire: 0
    }, config); // 防止重复创建

    if (!loadingElement) {
      createLoading(config.isRectLoading);
    } else if (prevIsRectLoading !== config.isRectLoading) {
      // loading 类型不同时，重新创建
      try {
        document.body.removeChild(loadingElement);
      } catch (e) {}

      createLoading(config.isRectLoading);
    } else {
      loadingElement.style.display = 'block';
    }

    prevIsRectLoading = config.isRectLoading; // 如果配置中有expire且大于0
    // 将会在到达时间后自动关闭，不用手动再关闭（不需要再close()）

    if (config.expire > 0) {
      loadingTimer = setTimeout(function () {
        Loading.close();
      }, config.expire);
    }

    var id = new Date().getTime();
    openIds.push(id);
    return id;
  };

  Loading.close = function (closeId) {
    if (!_string.canUseDOM) {
      return;
    } // 防止重复关闭


    if (loadingTimer) {
      clearTimeout(loadingTimer);
    }

    if (closeTimer) {
      clearTimeout(closeTimer);
    }

    if (!closeId) {
      openIds = [];
      closeIds = [];

      if (loadingElement) {
        loadingElement.style.display = 'none';
      }

      return;
    }

    closeIds.push(closeId); // 代码执行时，由于一些接口loading很短暂，而另一个接口还没有开始调用，
    // 只有closeId是不能真正解决一些特殊loading抖动情况，
    // 根据js的任务队列，我们加入setTimeout，只有主线程结束后才执行销毁

    closeTimer = setTimeout(function () {
      closeIds.forEach(function (id) {
        var index = openIds.indexOf(id);

        if (index >= 0) {
          openIds.splice(index, 1);
        }
      });

      if (!openIds.length) {
        closeIds = [];

        if (loadingElement) {
          loadingElement.style.display = 'none';
        }
      }
    });
  };

  return Loading;
}();

var _default = Loading;
exports.default = _default;