"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var __extends = void 0 && (void 0).__extends || function () {
  var _extendStatics = function extendStatics(d, b) {
    _extendStatics = Object.setPrototypeOf || {
      __proto__: []
    } instanceof Array && function (d, b) {
      d.__proto__ = b;
    } || function (d, b) {
      for (var p in b) {
        if (b.hasOwnProperty(p)) d[p] = b[p];
      }
    };

    return _extendStatics(d, b);
  };

  return function (d, b) {
    _extendStatics(d, b);

    function __() {
      this.constructor = d;
    }

    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
  };
}();

/**
 * 动态加入js 代码
 * @param code
 * @param url
 */
function loadScriptCode(code, url) {
  if (!code) {
    return;
  }

  var script = document.createElement('script');
  script.type = 'text/javascript';
  script.setAttribute('data-src', url);

  try {
    script.appendChild(document.createTextNode(code));
  } catch (ex) {
    // 兼容IE
    script.text = code;
  }

  document.body.appendChild(script);
}
/**
 * 动态创建和加载js文件，比如jquery
 */


var Script =
/** @class */
function (_super) {
  __extends(Script, _super);

  function Script() {
    var _this = _super !== null && _super.apply(this, arguments) || this;

    _this.loadSrc = function (isRefresh, preProps) {
      var _a = _this.props,
          url = _a.url,
          beforeCode = _a.beforeCode,
          cancelable = _a.cancelable,
          onRemove = _a.onRemove; // beforeCode

      if (isRefresh || preProps.beforeCode !== beforeCode) {
        // 这里只支持在url改变时，删除code, 并不对code进行判断来删除url的script， 原因是没有这种需求
        if (cancelable && preProps && preProps.beforeCode !== beforeCode) {
          var preCodeElement = document.querySelector("script[data-src='" + preProps.url + "']");

          if (preCodeElement && preCodeElement.remove) {
            preCodeElement.remove();
          }

          onRemove(true, preProps.url, preProps.attributes);
        }

        loadScriptCode(beforeCode, url);
      } // url 不变的情况下，每个组件只执行一次onLoad，onError


      if (isRefresh || url !== preProps.url) {
        var scriptsElement = document.querySelector("script[src='" + url + "']"); // 如果已经存在， 警告, 并终止执行

        if (scriptsElement) {
          // eslint-disable-next-line no-console
          console.error("Src: " + url + " has been created, the corresponding callback will no longer be executed, Please use it directly");
          return;
        }

        if (cancelable && preProps && url !== preProps.url) {
          var preScriptsElement = document.querySelector("script[src='" + preProps.url + "']");

          if (preScriptsElement && preScriptsElement.remove) {
            preScriptsElement.remove();
          }

          onRemove(false, preProps.url, preProps.attributes);
        }

        _this.createScript();
      }
    };

    return _this;
  }

  Script.prototype.componentDidMount = function () {
    this.loadSrc(true);
  };

  Script.prototype.componentDidUpdate = function (prevProps) {
    this.loadSrc(false, prevProps);
  };

  Script.prototype.componentWillUnmount = function () {
    var _a = this.props,
        cancelable = _a.cancelable,
        url = _a.url,
        attributes = _a.attributes,
        onRemove = _a.onRemove;

    if (cancelable && url) {
      var scriptsElement = document.querySelector("script[src='" + url + "']");

      if (scriptsElement && scriptsElement.remove) {
        scriptsElement.remove();
      }

      onRemove(false, url, attributes);
    }
  };

  Script.prototype.createScript = function () {
    var _a = this.props,
        url = _a.url,
        attributes = _a.attributes,
        onCreate = _a.onCreate,
        onLoad = _a.onLoad,
        onError = _a.onError;
    onCreate(url, attributes);
    var script = document.createElement('script');

    if (attributes) {
      Object.keys(attributes).forEach(function (prop) {
        script.setAttribute(prop, prop === 'async' || prop === 'crossOrigin' ? '' : attributes[prop]);
      });
    }

    script.src = url;

    script.onload = function () {
      onLoad(url, attributes);
    };

    script.onerror = function () {
      onError(url, attributes);
    };

    document.body.appendChild(script);
  };

  Script.prototype.render = function () {
    return null;
  };

  Script.defaultProps = {
    beforeCode: '',
    cancelable: true,
    attributes: {
      async: true
    },
    onCreate: function onCreate() {},
    onError: function onError() {},
    onLoad: function onLoad() {},
    onRemove: function onRemove() {}
  };
  return Script;
}(_react.default.Component);

var _default = Script;
exports.default = _default;