"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getPopupLocation;

var _constants = require("./constants");

var _get = require("../utils/get");

var _align = require("./align");

var __assign = void 0 && (void 0).__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) {
        if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
      }
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

function getTranslateX(_a, availSize, initTranslateX, popupLimitSpacing, scrollValue) {
  var left = _a.left,
      width = _a.width;
  var limit = initTranslateX ? 0 : popupLimitSpacing;

  if (width < availSize.width - limit * 2) {
    if (left - scrollValue.left < limit) {
      return {
        translateX: -left + scrollValue.left + limit,
        maxWidth: undefined
      };
    }

    if (left - scrollValue.left + width + limit > availSize.width) {
      return {
        translateX: availSize.width - (left + width + limit) + scrollValue.left,
        maxWidth: undefined
      };
    }
  } else {
    var maxWidth = availSize.width - limit * 2;
    return {
      translateX: availSize.width - (left + maxWidth + limit) + scrollValue.left,
      maxWidth: maxWidth
    };
  }

  return {
    translateX: initTranslateX,
    maxWidth: undefined
  };
}

function getTranslateY(_a, availSize, popupLimitSpacing, scrollValue) {
  var top = _a.top,
      height = _a.height;

  if (top - scrollValue.top < popupLimitSpacing) {
    return -top + scrollValue.top + popupLimitSpacing;
  }

  if (height < availSize.height && top - scrollValue.top + height + popupLimitSpacing > availSize.height) {
    return availSize.height - top + scrollValue.top - height - popupLimitSpacing;
  }

  return 0;
}
/**
 * 获取 popup 元素接下来的位置信息
 * @returns {*}
 */


function getPopupLocation(placement, rootEl, popupEl, initTranslateX, initTranslateY, rootToPopupSpacing, popupLimitSpacing, isTransformHorizontalDirection, hiddenArrow, isInsertNode) {
  var isHidden = false; // 判断body是否有隐藏, 并计算可用区域大小

  try {
    var bodyOverflowValue = (0, _get.getStyle)(document.body, 'overflow');
    isHidden = bodyOverflowValue === 'hidden';
  } catch (e) {}

  var rootInfo = (0, _get.getOffset)(rootEl, isHidden);
  var popInfo = (0, _get.getOffset)(popupEl, isHidden);
  var availSize = (0, _align.getAvailSize)(isHidden);
  var parentNodeInfo = {
    left: 0,
    top: 0
  };
  var leftPopupLimitSpacing = popupLimitSpacing;
  var topPopupLimitSpacing = popupLimitSpacing; // 悬浮框插入到文档流中

  if (isInsertNode) {
    var parentNode = rootEl.parentNode; // 判断父级元素是否定位，否则添加relative

    if ((0, _get.getStyle)(parentNode, 'position') === 'static') {
      parentNode.style.position = 'relative';
    }

    parentNodeInfo = (0, _get.getOffset)(parentNode); // 扩充popup临界点

    leftPopupLimitSpacing = -parentNodeInfo.left + leftPopupLimitSpacing;
    topPopupLimitSpacing = -parentNodeInfo.top + topPopupLimitSpacing;
  }

  var scrollValue = {
    left: 0,
    top: 0
  };

  if (isHidden) {
    scrollValue = (0, _get.getScrollValue)();
  }

  var params = {
    rootInfo: rootInfo,
    popInfo: popInfo,
    isTransformHorizontalDirection: isTransformHorizontalDirection,
    rootToPopupSpacing: rootToPopupSpacing,
    hiddenArrow: hiddenArrow,
    translateX: initTranslateX,
    translateY: initTranslateY,
    scrollTop: scrollValue.top,
    scrollLeft: scrollValue.left,
    isInsertNode: isInsertNode
  };

  if (placement === _constants.LocationType.Top.value) {
    var result = (0, _align.getTopLocationInfo)(params);
    var translateX = getTranslateX(__assign(__assign({}, result), {
      left: isInsertNode ? parentNodeInfo.left + result.left : result.left
    }), availSize, initTranslateX, popupLimitSpacing, scrollValue).translateX;

    if (result.top - scrollValue.top < topPopupLimitSpacing) {
      return (0, _align.getBottomLocationInfo)(__assign(__assign({}, params), {
        translateX: translateX
      }));
    }

    if (!translateX) {
      return result;
    }

    return (0, _align.getTopLocationInfo)(__assign(__assign({}, params), {
      translateX: translateX
    }));
  }

  if (placement === _constants.LocationType.Bottom.value) {
    var result = (0, _align.getBottomLocationInfo)(params);
    var translateX = getTranslateX(__assign(__assign({}, result), {
      left: isInsertNode ? parentNodeInfo.left + result.left : result.left
    }), availSize, initTranslateX, popupLimitSpacing, scrollValue).translateX; // 已经超出底部

    if ( // 判断底部是否能装下悬浮框
    (isInsertNode && result.top - scrollValue.top + popInfo.height - topPopupLimitSpacing > availSize.height || result.top - scrollValue.top + result.height + topPopupLimitSpacing > availSize.height) && // 判断rootElement上半部分是否能装下悬浮框
    rootInfo.top - (rootToPopupSpacing + popInfo.height) > topPopupLimitSpacing) {
      return (0, _align.getTopLocationInfo)(__assign(__assign({}, params), {
        translateX: translateX
      }));
    } // 正常情况直接返回


    if (!translateX) {
      return result;
    }

    return (0, _align.getBottomLocationInfo)(__assign(__assign({}, params), {
      translateX: translateX
    }));
  }

  if (placement === _constants.LocationType.Left.value) {
    var result = (0, _align.getLeftLocationInfo)(params);
    var translateY = getTranslateY(result, availSize, topPopupLimitSpacing, scrollValue);

    if (result.left - scrollValue.left < leftPopupLimitSpacing) {
      return (0, _align.getRightLocationInfo)(__assign(__assign({}, params), {
        translateY: translateY
      }));
    }

    if (!translateY) {
      return result;
    }

    return (0, _align.getLeftLocationInfo)(__assign(__assign({}, params), {
      translateY: translateY
    }));
  }

  if (placement === _constants.LocationType.Right.value) {
    var result = (0, _align.getRightLocationInfo)(params);
    var translateY = getTranslateY(result, availSize, topPopupLimitSpacing, scrollValue);

    if ( // 判断右半部是否能装下悬浮框
    (isInsertNode && result.left - scrollValue.left + popInfo.width - leftPopupLimitSpacing > availSize.width || result.left - scrollValue.left + result.width + leftPopupLimitSpacing > availSize.width) && // 判断rootElement左半部分是否能装下悬浮框
    rootInfo.left - (rootToPopupSpacing + popInfo.width) > leftPopupLimitSpacing) {
      return (0, _align.getLeftLocationInfo)(__assign(__assign({}, params), {
        translateY: translateY
      }));
    } // 不再重新计算


    if (!translateY) {
      return result;
    }

    return (0, _align.getRightLocationInfo)(__assign(__assign({}, params), {
      translateY: translateY
    }));
  }

  if (placement === _constants.LocationType.LeftTop.value || placement === _constants.LocationType.LeftBottom.value) {
    var isLeftTop = placement === _constants.LocationType.LeftTop.value;
    var result = isLeftTop ? (0, _align.getLeftTopLocationInfo)(params) : (0, _align.getLeftBottomLocationInfo)(params);
    var translateY = getTranslateY(result, availSize, topPopupLimitSpacing, scrollValue);

    if (result.left - scrollValue.left < leftPopupLimitSpacing) {
      return isLeftTop ? (0, _align.getRightTopLocationInfo)(__assign(__assign({}, params), {
        translateY: translateY
      })) : (0, _align.getRightBottomLocationInfo)(__assign(__assign({}, params), {
        translateY: translateY
      }));
    }

    if (!translateY) {
      return result;
    }

    return isLeftTop ? (0, _align.getLeftTopLocationInfo)(__assign(__assign({}, params), {
      translateY: translateY
    })) : (0, _align.getLeftBottomLocationInfo)(__assign(__assign({}, params), {
      translateY: translateY
    }));
  }

  if (placement === _constants.LocationType.RightTop.value || placement === _constants.LocationType.RightBottom.value) {
    var isRightTop = placement === _constants.LocationType.RightTop.value;
    var result = isRightTop ? (0, _align.getRightTopLocationInfo)(params) : (0, _align.getRightBottomLocationInfo)(params);
    var translateY = getTranslateY(result, availSize, topPopupLimitSpacing, scrollValue);

    if ( // 判断右半部是否能装下悬浮框
    (isInsertNode && result.left - scrollValue.left + popInfo.width - leftPopupLimitSpacing > availSize.width || result.left - scrollValue.left + result.width + leftPopupLimitSpacing > availSize.width) && // 判断rootElement左半部分是否能装下悬浮框
    rootInfo.left - (rootToPopupSpacing + popInfo.width) > leftPopupLimitSpacing) {
      return isRightTop ? (0, _align.getLeftTopLocationInfo)(__assign(__assign({}, params), {
        translateY: translateY
      })) : (0, _align.getLeftBottomLocationInfo)(__assign(__assign({}, params), {
        translateY: translateY
      }));
    }

    if (!translateY) {
      return result;
    }

    return isRightTop ? (0, _align.getRightTopLocationInfo)(__assign(__assign({}, params), {
      translateY: translateY
    })) : (0, _align.getRightBottomLocationInfo)(__assign(__assign({}, params), {
      translateY: translateY
    }));
  }

  if (placement === _constants.LocationType.TopLeft.value || placement === _constants.LocationType.TopRight.value) {
    var isTopLeft = placement === _constants.LocationType.TopLeft.value;
    var result = isTopLeft ? (0, _align.getTopLeftLocationInfo)(params) : (0, _align.getTopRightLocationInfo)(params);
    var translateX = getTranslateX(__assign(__assign({}, result), {
      left: isInsertNode ? parentNodeInfo.left + result.left : result.left
    }), availSize, initTranslateX, popupLimitSpacing, scrollValue).translateX;

    if (result.top - scrollValue.top < topPopupLimitSpacing) {
      return isTopLeft ? (0, _align.getBottomLeftLocationInfo)(__assign(__assign({}, params), {
        translateX: translateX
      })) : (0, _align.getBottomRightLocationInfo)(__assign(__assign({}, params), {
        translateX: translateX
      }));
    }

    if (!translateX) {
      return result;
    }

    return isTopLeft ? (0, _align.getTopLeftLocationInfo)(__assign(__assign({}, params), {
      translateX: translateX
    })) : (0, _align.getTopRightLocationInfo)(__assign(__assign({}, params), {
      translateX: translateX
    }));
  }

  if (placement === _constants.LocationType.BottomLeft.value || placement === _constants.LocationType.BottomRight.value) {
    var isBottomLeft = placement === _constants.LocationType.BottomLeft.value;
    var result = isBottomLeft ? (0, _align.getBottomLeftLocationInfo)(params) : (0, _align.getBottomRightLocationInfo)(params);

    var _a = getTranslateX(__assign(__assign({}, result), {
      left: isInsertNode ? parentNodeInfo.left + result.left : result.left
    }), availSize, initTranslateX, popupLimitSpacing, scrollValue),
        translateX = _a.translateX,
        maxWidth = _a.maxWidth;

    if ( // 判断底部是否能装下悬浮框
    (isInsertNode && result.top - scrollValue.top + popInfo.height - topPopupLimitSpacing > availSize.height || result.top - scrollValue.top + result.height + topPopupLimitSpacing > availSize.height) && // 判断rootElement上半部分是否能装下悬浮框
    rootInfo.top - (rootToPopupSpacing + popInfo.height) > topPopupLimitSpacing) {
      return isBottomLeft ? (0, _align.getTopLeftLocationInfo)(__assign(__assign({}, params), {
        translateX: translateX
      })) : (0, _align.getTopRightLocationInfo)(__assign(__assign({}, params), {
        translateX: translateX
      }));
    } // 正常情况直接返回


    if (!translateX) {
      return result;
    }

    var value = isBottomLeft ? (0, _align.getBottomLeftLocationInfo)(__assign(__assign({}, params), {
      translateX: translateX
    })) : (0, _align.getBottomRightLocationInfo)(__assign(__assign({}, params), {
      translateX: translateX
    })); // 限制宽度

    value.maxWidth = maxWidth;
    return value;
  }

  throw new TypeError('Support placement: Top, TopLeft TopRight, Bottom, BottomLeft, BottomRight, Right, RightTop, RightBottom, Left, LeftTop, LeftBottom');
}