"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.EasingTypes = exports.EventTypes = exports.ModeTypes = void 0;

var _selfEvent = require("../selfEvent");

var _tween = _interopRequireDefault(require("./tween"));

var _animationFrame = require("../animationFrame");

var _constants = require("./constants");

var __assign = void 0 && (void 0).__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) {
        if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
      }
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

var ModeTypes = (0, _constants.getModeTypes)();
exports.ModeTypes = ModeTypes;
var EventTypes = (0, _constants.getEventTypes)();
exports.EventTypes = EventTypes;
var EasingTypes = (0, _constants.getEasingTypes)();
exports.EasingTypes = EasingTypes;

var Animation =
/** @class */
function () {
  function Animation(options) {
    var _this = this;

    this.initProperties = {};
    this.nextProperties = {};
    this.targetProperties = {};
    this.runTime = 0;

    this.cancelTimer = function () {
      (0, _animationFrame.cancelAnimationFrame)(_this.animationTimer);
      clearTimeout(_this.sleepTimer);
    };
    /**
     * 任务回调
     * @param eventType
     */


    this.emitSelfEvent = function (eventType) {
      var params = {
        currentValue: __assign({}, _this.nextProperties),
        targetValue: __assign({}, _this.targetProperties),
        initValue: __assign({}, _this.initProperties)
      };

      _this.selfEvent.emit(eventType, params);

      _this.selfEvent.emit(EventTypes.always, __assign(__assign({}, params), {
        eventType: eventType
      }));
    };
    /**
     * 循环， 并触发回调
     * @param tweenCallback
     */


    this.run = function (tweenCallback) {
      // 停止执行回调
      if (_this.phase === EventTypes.abort || _this.phase === EventTypes.sleep) {
        return;
      }

      var duration = _this.options.duration;
      var endTime = Date.now(); // 累加时间

      _this.runTime += endTime - _this.startTime; // 缓存这次执行的时间

      _this.startTime = endTime; // 循环Properties， 并逐个执行

      for (var prop in _this.initProperties) {
        _this.nextProperties[prop] = tweenCallback(_this.runTime, _this.initProperties[prop], _this.targetProperties[prop] - _this.initProperties[prop], duration);
      }

      _this.emitSelfEvent(EventTypes.progress); // 没有到指定时间


      if (_this.runTime <= duration) {
        _this.animationTimer = (0, _animationFrame.requestAnimationFrame)(_this.run.bind(_this, tweenCallback));
      } else {
        // 动画结束
        _this.cancelTimer();

        _this.emitSelfEvent(EventTypes.complete);
      }
    };

    this.progress = function () {
      var _a = _this.options,
          mode = _a.mode,
          easing = _a.easing;

      if (_this.animationTimer) {
        (0, _animationFrame.cancelAnimationFrame)(_this.animationTimer);
      }

      try {
        _this.run(ModeTypes.Linear === mode ? _tween.default.Linear : _tween.default[mode][easing]);
      } catch (e) {
        _this.selfEvent.emit(EventTypes.error, e);
      }
    };
    /**
     * 监听事件
     * @param eventType
     * @param callback
     */


    this.on = function (eventType, callback) {
      _this.selfEvent.on(eventType, callback);
    };
    /**
     * 开始动画
     * @param initProperties
     * @param targetProperties
     * @param options
     */


    this.start = function (initProperties, targetProperties, options) {
      _this.initProperties = __assign(__assign({}, _this.initProperties), initProperties);
      _this.targetProperties = __assign(__assign({}, _this.targetProperties), targetProperties);

      for (var prop in _this.initProperties) {
        // init 没有对应的target prop, 将抛出异常
        if (!(prop in _this.targetProperties)) {
          _this.selfEvent.emit(EventTypes.error, 'The expected field does not match the original field');

          return;
        }
      }

      if (options) {
        _this.options = __assign(__assign({}, _this.options), options);
      }

      _this.startTime = Date.now();
      _this.runTime = 0;
      _this.phase = null;

      _this.cancelTimer();

      _this.emitSelfEvent(EventTypes.start);

      _this.progress();
    };
    /**
     * 延迟动画
     * @param expire 单位ms
     */


    this.sleep = function (expire) {
      if (expire <= 0) {
        _this.selfEvent.emit(EventTypes.error, 'sleep expire must be greater than 0');

        return;
      }

      _this.phase = EventTypes.sleep;

      _this.emitSelfEvent(EventTypes.sleep); // 开启定时器，并到expire后执行


      _this.sleepTimer = setTimeout(function () {
        _this.cancelTimer();

        _this.phase = null; // 重置开始时间

        _this.startTime = Date.now();

        _this.progress();
      }, expire);
    };
    /**
     * 终止动画
     */


    this.abort = function () {
      _this.phase = EventTypes.abort;

      _this.cancelTimer();

      _this.runTime = 0;

      _this.emitSelfEvent(EventTypes.abort);
    };
    /**
     * 删除动画
     */


    this.cancel = function () {
      _this.cancelTimer();

      _this.selfEvent = null;
      _this.phase = EventTypes.abort;
      _this.runTime = 0;
    };

    this.options = __assign(__assign({}, Animation.defaultOptions), options);
    this.selfEvent = new _selfEvent.SelfEvent();
  }

  Animation.defaultOptions = {
    duration: 300,
    easing: 'easeInOut',
    mode: 'Quadratic'
  };
  return Animation;
}();

var _default = Animation;
exports.default = _default;