"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var __assign = void 0 && (void 0).__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) {
        if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
      }
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

var ClientCache =
/** @class */
function () {
  // 实例初始化
  function ClientCache(options) {
    var _this = this; // 默认配置


    this.config = {
      version: '2021',
      // 默认缓存1天
      delayDay: 1,
      storageMethod: window && window.localStorage ? window.localStorage : null
    };
    /**
     * 获取时间戳
     * @param delayDay 当前时间戳+delayDay = 某个时间戳
     * @private
     */

    this._getTimestamp = function (delayDay) {
      var currentTimestamp = Date.now();

      if (!delayDay || delayDay <= 0) {
        return currentTimestamp;
      } // 当前时间戳+delayDay = 某个时间戳


      return currentTimestamp + 60 * 60 * 24 * 1000 * delayDay;
    };
    /**
     * 保存数据，并设置版本和时间戳
     * @param key
     * @param namespace
     * @param data
     * @param delayDay
     */


    this.setData = function (key, namespace, data, delayDay) {
      if (!_this.config.storageMethod || !key || !namespace || !data) {
        return false;
      }

      var _a = _this.config,
          storageMethod = _a.storageMethod,
          logger = _a.logger;

      try {
        // 设置版本和过期时间
        storageMethod.setItem(namespace + "_" + key, JSON.stringify({
          version: _this.config.version,
          expires: _this._getTimestamp(delayDay || _this.config.delayDay),
          data: data
        }));
      } catch (error) {
        // 出现异常（QUOTA_EXCEEDED_ERR、空指针等）, 清除所有缓存
        storageMethod.clear(); // 如果有日志，将回调日志

        if (typeof logger === 'function') {
          logger({
            response: error ? error.stack : error
          });
        }

        return false;
      }

      return true;
    };
    /**
     * 获取数据，把在setData设置版本和时间戳清洗掉，直接返回data
     * @param key
     * @param namespace
     */


    this.getData = function (key, namespace) {
      if (!_this.config.storageMethod || !key || !namespace) {
        return null;
      }

      var storageMethod = _this.config.storageMethod;
      var itemKey = namespace + "_" + key;
      var value = storageMethod.getItem(itemKey);

      if (!value) {
        return null;
      }

      var _a = _this.config,
          logger = _a.logger,
          version = _a.version;

      try {
        var cacheData = JSON.parse(value); // 版本不同

        if (!cacheData.version || cacheData.version !== version) {
          // 清除当前缓存
          storageMethod.removeItem(itemKey);
          return null;
        } // 已过期


        var currentTimestamp = _this._getTimestamp();

        if (!cacheData.expires || currentTimestamp > cacheData.expires) {
          // 清除当前缓存
          storageMethod.removeItem(itemKey);
          return null;
        } // 返回之前传的数据，把版本和过期时间清洗掉


        return cacheData.data;
      } catch (error) {
        // 清除当前缓存
        storageMethod.removeItem(itemKey); // 如果有日志，将回调日志

        if (typeof logger === 'function') {
          logger({
            response: error ? error.stack : error
          });
        }

        return null;
      }
    };

    this.config = __assign(__assign({}, this.config), options);
  }

  return ClientCache;
}();

var _default = ClientCache;
exports.default = _default;