"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getClientPosition = getClientPosition;
exports.getElementSize = getElementSize;
exports.getClientSize = getClientSize;
exports.getOffset = getOffset;
exports.getScrollValue = getScrollValue;
exports.getStyle = getStyle;
exports.getStyleValue = getStyleValue;

function getClientPosition(elem) {
  if (!elem) {
    return {
      left: 0,
      top: 0
    };
  }

  var x;
  var y;
  var doc = elem.ownerDocument;

  if (!doc) {
    return {
      left: 0,
      top: 0
    };
  }

  var body = doc.body;
  var docElem = doc.documentElement;
  var box = elem.getBoundingClientRect();
  x = box.left;
  y = box.top;
  x -= docElem.clientLeft || body.clientLeft || 0;
  y -= docElem.clientTop || body.clientTop || 0;
  return {
    left: x,
    top: y
  };
}

function getElementSize(elem, isRemoveBorder) {
  if (!elem) {
    return {
      width: 0,
      height: 0
    };
  }

  if (isRemoveBorder) {
    return {
      width: elem.clientWidth,
      height: elem.clientHeight
    };
  }

  return {
    width: elem.offsetWidth,
    height: elem.offsetHeight
  };
}

function getClientSize() {
  // TODO:: window.innerWidth = document.documentElement.clientWidth + scrollWidth，
  //  innerWidth在Chrome模拟器上有问题， 可手动缩小屏幕进行解决， 或者使用兼容的写法
  //  https://stackoverflow.com/questions/36297612/window-innerwidth-in-chromes-device-mode
  //  可使用window.visualViewport.width进行代替，但是这个兼容性很差
  return {
    width: window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth,
    height: window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight
  };
}

function getOffset(elem, ignoreScroll) {
  if (!elem) {
    return {
      left: 0,
      top: 0,
      width: 0,
      height: 0
    };
  }

  var _a = getClientPosition(elem),
      left = _a.left,
      top = _a.top;

  var _b = getElementSize(elem),
      width = _b.width,
      height = _b.height;

  if (ignoreScroll) {
    return {
      left: left,
      top: top,
      offsetLeft: elem.offsetLeft,
      offsetTop: elem.offsetTop,
      width: width,
      height: height
    };
  }

  var scrollValue = getScrollValue(); // 需要处理滚动

  return {
    left: left + scrollValue.left,
    top: top + scrollValue.top,
    offsetLeft: elem.offsetLeft,
    offsetTop: elem.offsetTop,
    width: width,
    height: height
  };
}

function getScrollValue() {
  return {
    left: document.body.scrollLeft + document.documentElement.scrollLeft,
    top: document.body.scrollTop + document.documentElement.scrollTop
  };
}
/**
 * 获取执行元素的属性值
 * @param element
 * @param attr
 * @returns {string}
 */


function getStyle(element, attr) {
  try {
    var style = getComputedStyle ? getComputedStyle(element, null) : document.defaultView.getComputedStyle(element, null);
    return attr ? style[attr] : style;
  } catch (e) {
    return '0';
  }
}

function getStyleValue(element, attr) {
  var str = getStyle(element, attr);
  var value = parseFloat(str);
  return isNaN(value) ? 0 : value;
}