"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.SelfEvent = void 0;

var SelfEvent =
/** @class */
function () {
  function SelfEvent() {
    this.listeners = [];
  }

  SelfEvent.getInstance = function () {
    // 单例模式
    return SelfEvent.instance || (SelfEvent.instance = new SelfEvent());
  };

  SelfEvent.prototype._findPrevListener = function (eventName) {
    var l = this.listeners.length;

    for (var i = 0, listener = void 0; i < l; i++) {
      listener = this.listeners[i];

      if (listener && listener.eventName === eventName) {
        return {
          listener: listener,
          index: i
        };
      }
    }

    return {
      listener: null,
      index: null
    };
  };

  SelfEvent.prototype._addListener = function (eventName, handler, context) {
    var listener = this._findPrevListener(eventName).listener;

    if (listener) {
      var handlers = listener.handlers;

      if (handlers) {
        var size = handlers.length; // 判断已经有相同的handler就禁止再添加

        for (var i = 0, h = void 0; i < size; i++) {
          h = handlers[i];

          if (h.context === context && h.handler === handler) {
            return;
          }
        }
      }

      listener.handlers.push({
        context: context,
        handler: handler
      });
    } else {
      this.listeners.push({
        eventName: eventName,
        handlers: [{
          context: context,
          handler: handler
        }]
      });
    }
  };

  SelfEvent.prototype._removeListener = function (index) {
    this.listeners.splice(index, 1);
  }; // 触发自定义事件


  SelfEvent.prototype.emit = function (eventName, params, context) {
    if (!eventName) {
      return;
    }

    var _a = this._findPrevListener(eventName),
        listener = _a.listener,
        index = _a.index;

    if (!listener) {
      return;
    }

    var handlers = listener.handlers;

    if (!handlers.length) {
      this._removeListener(index);

      return;
    }

    for (var i = 0, l = handlers.length, item = void 0; i < l; i++) {
      item = handlers[i]; // 保证函数和上下文相同，或者没有上下文

      if (!context || !item.context || item.context === context) {
        item.handler(params);
      }
    }
  }; // 监听事件变化，触发回调


  SelfEvent.prototype.on = function (eventName, handler, context) {
    var _this = this;

    this._addListener(eventName, handler, context); // 返回清除函数，执行清除监听


    return function () {
      _this.off(eventName, handler, context);
    };
  };

  SelfEvent.prototype.off = function (eventName, handler, context) {
    var _a = this._findPrevListener(eventName),
        listener = _a.listener,
        index = _a.index;

    if (!listener) {
      return;
    }

    var handlers = listener.handlers; // 当没有指定清除哪个具体函数， 就是清除当前元素所有监听事件

    if (!handler || !handlers.length) {
      this._removeListener(index);

      return;
    }

    for (var i = 0, l = handlers.length, item = void 0; i < l; i++) {
      item = handlers[i]; // 保证函数和上下文相同，或者没有上下文

      if (item.handler === handler && (!context || !item.context || item.context === context)) {
        // 如果已经是最后一条数据了，也清除当前元素监听
        if (l < 2) {
          this._removeListener(index);

          return;
        }

        handlers.splice(i, 1);
        break;
      }
    }
  };

  SelfEvent.prototype.offAll = function () {
    this.listeners = [];
  };

  return SelfEvent;
}();

exports.SelfEvent = SelfEvent;

var _default = SelfEvent.getInstance();

exports.default = _default;