"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isBlank = isBlank;
exports.isAllBlank = isAllBlank;
exports.isAllFull = isAllFull;
exports.isUndef = isUndef;
exports.isEmptyObject = isEmptyObject;
exports.isObject = isObject;
exports.isArray = isArray;
exports.isDate = isDate;
exports.isEmptyArray = isEmptyArray;
exports.hasHorizontalScrollbar = hasHorizontalScrollbar;
exports.hasVerticalScrollbar = hasVerticalScrollbar;
exports.hasScrollbar = hasScrollbar;
exports.isTouchScreen = isTouchScreen;
exports.canUseDOM = void 0;

var _get = require("./get");

var _getScrollBarSize = require("./getScrollBarSize");

function isBlank(val) {
  return isUndef(val) || val === '' || val.toString().trim() === '';
}

function isAllBlank() {
  var args = [];

  for (var _i = 0; _i < arguments.length; _i++) {
    args[_i] = arguments[_i];
  }

  return args.every(function (arg) {
    return isBlank(arg);
  });
}

function isAllFull() {
  var args = [];

  for (var _i = 0; _i < arguments.length; _i++) {
    args[_i] = arguments[_i];
  }

  return args.every(function (arg) {
    return !isBlank(arg);
  });
}

function isUndef(val) {
  return val === null || typeof val === 'undefined';
}

function isEmptyObject(obj) {
  return !obj || Object.keys(obj).length <= 0;
}

var objToString = Object.prototype.toString;

function isObject(obj) {
  return objToString.call(obj) === '[object Object]';
}

function isArray(obj) {
  return objToString.call(obj) === '[object Array]';
}

function isDate(value) {
  return Object.prototype.toString.call(value) === '[object Date]';
}

function isEmptyArray(obj) {
  return !isArray(obj) || obj.length <= 0;
}

function hasHorizontalScrollbar(prevScrollSize) {
  var paddingBottom = (0, _get.getStyleValue)(window.document.body, 'paddingBottom');
  var scrollSize = prevScrollSize || (0, _getScrollBarSize.getScrollBarSize)();
  return paddingBottom !== 0 && paddingBottom === scrollSize || document.body.scrollWidth > (window.innerWidth || document.documentElement.clientWidth);
}

function hasVerticalScrollbar(prevScrollSize) {
  var paddingRight = (0, _get.getStyleValue)(window.document.body, 'paddingRight');
  var scrollSize = prevScrollSize || (0, _getScrollBarSize.getScrollBarSize)();
  return paddingRight !== 0 && paddingRight === scrollSize || document.body.scrollHeight > (window.innerHeight || document.documentElement.clientHeight);
}

function hasScrollbar(prevScrollSize) {
  var scrollSize = prevScrollSize || (0, _getScrollBarSize.getScrollBarSize)();
  return hasVerticalScrollbar(scrollSize) || hasHorizontalScrollbar(scrollSize);
}

var canUseDOM = !!(typeof window !== 'undefined' && window.document && window.document.createElement);
exports.canUseDOM = canUseDOM;

function isTouchScreen() {
  return canUseDOM && 'ontouchstart' in document.documentElement;
}